package seat;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import seat.ui.views.AlgorithmPreference;
import seat.ui.views.DecorationPreference;
import seat.ui.views.IconPreference;
import seat.ui.views.TracePreference;
import seat.ui.views.UtilityPreference;

/**
 * The main plugin class for trace exploration.
 */
public class SeatPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static SeatPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	private SeatExtra extra=new SeatExtra(); // data will be set in OpenTraceWizard, used in parsing
	
	/**
	 * The constructor.
	 */
	public SeatPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;
		try {
			resourceBundle= ResourceBundle.getBundle("org.kbre.seat.SeatPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * Returns the shared instance.
	 */
	public static SeatPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= SeatPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#initializeDefaultPreferences(org.eclipse.jface.preference.IPreferenceStore)
	 */
	protected void initializeDefaultPreferences(IPreferenceStore store) {
		// TODO Auto-generated method stub
		TracePreference.setDefaultConfig(store);
		DecorationPreference.setDefaultConfig(store);
		IconPreference.setDefaultConfig(store);
		UtilityPreference.setDefaultConfig(store);
		AlgorithmPreference.setDefaultConfig(store);
		super.initializeDefaultPreferences(store);
	}	
	
	public static ImageDescriptor getImageDescriptor(String iconName) {
		String iconPath="icons/";
		try {
			URL pluginUrl=getDefault().getDescriptor().getInstallURL();
			URL url=new URL(pluginUrl,iconPath+iconName);
			return ImageDescriptor.createFromURL(url);			
		} catch (MalformedURLException e) {
			return ImageDescriptor.getMissingImageDescriptor();
		}		
	}	
	
	/**
	 * Returns internal structure that hold temporary information data.
	 * @return 
	 */
	public SeatExtra getExtra() {
		return extra;
	}

}
