/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: Algorithm.java									 	 *	
 * Package: algorithms										     *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.model.algorithms;

import seat.model.core.dynamic.Trace;
import seat.ui.algorithm.IAlgorithm;
import seat.ui.algorithm.Parameter;
//May 5, 2004 Eric, for all algorithms that need to process single node, implements ICompressNode
//All algorithms will be modified to have inherited two methods apply(Trace trace)/undo(Trace trace).
//So those two interfaces are added,  

public abstract class Algorithm implements IAlgorithm
{
	// each algorithm should declare an unique integer index used to identify
	// itself in the BITS of a node. Cause should be taken not to use a previously 
	// occupied index. So an algorithm needs to document its index!! 
	// User indice start from 5; 0-4 are resevered for internal use, 0-for those 
	// without a correct index. 1 for filter, 2-utility, 
	public static int index=0;

	/**
	 * Initialization for an alogirthm. Subclasses may re-implement init() if they need
	 * further parameters, but MUST call super.init() first!! 
	 */
	public void init(Parameter[] parameters) {
		index=getIndex(parameters[0].getDefaultData());
		// TODO Auto-generated method stub
	}
	
	private int getIndex(String para0) {
		int unid;
		try {
			unid=Integer.parseInt(para0);
		} catch (NumberFormatException nfe) { 
			return 0;
		}
		if ( (unid<0)||(unid>=32))
			return 0;
		else 
			return unid;		
	}	
	
	abstract public void apply(Trace trace);
	abstract public void undo(Trace trace);
}