/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: IgnoreRep.java							 	 		 *	
 * Package: algorithms										     *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/

package seat.model.algorithms;

import java.util.Enumeration;
import java.util.Hashtable;

import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;
import seat.ui.algorithm.ICompressNode;
import seat.ui.algorithm.Parameter;
//May 5, 2004 Eric, for all algorithms that need to process single node, implements ICompressNode
//All algorithms will be modified to have inherited two methods apply(Trace trace)/undo(Trace trace).
//So those two interfaces are added, previuos process interface is keeped.    

public class IgnoreRep extends Algorithm implements ICompressNode {
	private static Hashtable table;
	// Global table for computing new certificates
	private static Hashtable visitedCompUnits; // Visited comprehenison units

	public void apply(Trace trace) {
		table = new Hashtable();   // donot put in init, becasue undo doesn't want them cleared
		visitedCompUnits = new Hashtable();
		Node node = ((TreeTrace) trace).getRoot();
		Criteria.ignoreRepetitions=true; // add by Eric
		ignoreRepHelper(node);
	}

	public void undo(Trace trace) {
		Node node = ((TreeTrace) trace).getRoot();
		Criteria.ignoreRepetitions=false; // add by Eric
		restoreRepHelper(node);
	}

	/**
	 * @param node node that will undergo the changes 
	 */

	public void process(Node node) {
		table = new Hashtable();
		visitedCompUnits = new Hashtable();
		Criteria.ignoreRepetitions=true; // add by Eric
		ignoreRepHelper(node);
	}

	/** Helps perfoming the ignore-repetition procedure. */

	private static void ignoreRepHelper(Node node) {
		if (node.isLeaf())
			return;

		for (Enumeration e = node.getChildren(); e.hasMoreElements();) {
			Node child = (Node) e.nextElement();

			if (!visitedCompUnits.containsKey(child)) {
				ignoreRepHelper(child);
				visitedCompUnits.put(child, "");
			}
		}

		String sig = node.computeSignature();

		if (table.containsKey(sig)) {
			int cert = Integer.parseInt((String) table.get(sig));
			node.setCertificate(cert);
			node.setState(2);

			System.out.println(
				"Enter: " + node.getLabel() + " " + node.getState());
		} else {
			table.put(sig, String.valueOf(node.getCertificate()));
		}
	}

	public void unprocess(Node node) {
		Criteria.ignoreRepetitions=false; // add by Eric
		restoreRepHelper(node);
	}

	/** Helps perfoming the ignore-order procedure. */

	private static void restoreRepHelper(Node node) {
		if (node.getState() == 2) {
			node.setCertificate(TreeTrace.getCertificate(node));
			node.setState(0);
		}

		for (Enumeration e = node.getChildren(); e.hasMoreElements();)
			restoreRepHelper((Node) e.nextElement());
	}

}