/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: LimithDepth.java							 	 		 *	
 * Package: algorithms										     *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/

package seat.model.algorithms;

import java.util.Enumeration;

import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;
import seat.ui.algorithm.*;
//May 5, 2004 Eric, for all algorithms that need to process single node, implements ICompressNode
//All algorithms will be modified to have inherited two methods apply(Trace trace)/undo(Trace trace).
//So those two interfaces are added, previuos process interface is keeped.  

// Also, if the node is to be hide if it exists at a deep level, the the node, will also be 
// hidden at upper level.--bugs or not. if yes, need to filter at ui level instead of ctf model 
public class LimitDepth extends Algorithm implements ICompressNode {

	private static int depth;
	// each algorithm should declare an unique integer index used to identify
	// itself in the BITS of a node. Cause should be taken not to use a previously 
	// occupied index. So an algorithm needs to document its index!! 
	// This index starts from 0; 

	public void apply(Trace trace) {
		//	   depth = Criteria.depth;
		//or 	init()
		Node node = ((TreeTrace) trace).getRoot();
		limitDepthHelper(node, 0);
	}

	public void undo(Trace trace) {
		Node node = ((TreeTrace) trace).getRoot();
		restoreDepthHelper(node);
	}

	/**
	 * @param node node that will undergo the changes 
	 */
	// it's better init be called within process
	// Repalce by an uniform init(Parameters[]), on May 20, 2003
	//  public void init(int dep) {
	//  	depth=dep;  
	//  }
	public void process(Node node) {
		//  	depth = Criteria.depth;
		// or 	init()
		limitDepthHelper(node, depth);
	}

	/** Helps perfoming the ignore-order procedure. */

	private static void limitDepthHelper(Node node, int level) {
		if (level > depth) {
			node.hide(index);
			return;
		}

		for (Enumeration e = node.getChildren(); e.hasMoreElements();)
			limitDepthHelper((Node) e.nextElement(), level + 1);
	}

	public void unprocess(Node node) {
		restoreDepthHelper(node);
	}

	/** Helps perfoming the ignore-order procedure. */

	private static void restoreDepthHelper(Node node) {
		if (node.isHidden()) {

			node.show(index);
		}

		for (Enumeration e = node.getChildren(); e.hasMoreElements();)
			restoreDepthHelper((Node) e.nextElement());
	}

	/* (non-Javadoc)
	 * @see seat.ui.algorithm.IAlgorithm#init(seat.ui.algorithm.Parameter[])
	 */
	public void init(Parameter[] parameters) {
		super.init(parameters);
		// TODO Auto-generated method stub
		for (int i = 0; i < parameters.length; i++)
			if (parameters[i].getID().equalsIgnoreCase("seat.depth")) {
				try {
					depth = Integer.parseInt(parameters[i].getCurrent());
				} catch (NumberFormatException nfe) {
					depth=0;
					return;
				}
				break;
			}
	}
}