/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: Pattern.java								 			 *	
 * Package: core.dynamic										 *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.model.core.dynamic;

import java.util.Enumeration;
import java.util.Vector;

public class Pattern 
{
  private String description;     // Pattern description
  private Vector pttOccurrences;  // Occurrences of this patterns in different traces

  /** Creates a pattern instance 
   *
   * @param description pattern description
   *
   */
    
  public Pattern(String description) 
  {
 	 this.description = description;
 	 this.pttOccurrences = new Vector();
  }

  /** Creates a pattern instance with no description */
    
  public Pattern() 
  {
 	 this.description = "";
 	 this.pttOccurrences = new Vector();
  }

  /** Sets the pattern description
   *
   * @param description description of the pattern
   *
   */

  public void setDescription(String description)
  {
  	 this.description = description;
  }

  /** Returns pattern description
   *
   * @return pattern description
   *
   */

  public String getDescription()
  {
  	 return description;
  }

  /** Adds a pattern occurrence
   *
   * @param pttOccurrence pattern occurrence to add to this pattern
   *
   */

  public void addPatternOccurrence(PatternOccurrence pttOccurrence)
  {
  	 pttOccurrences.addElement(pttOccurrence);
  }

  /** Removes a pattern occurrence
   *
   * @param pttOccurrence pattern occurrence to remove from this pattern
   *
   */

  public void removePatternOccurrence(PatternOccurrence pttOccurrence)
  {
  	 pttOccurrences.removeElement(pttOccurrence);
  }
  
  /** Returns all pattern occurrences
   *
   * @return all pattern occurrences of this pattern
   *
   */

  public Enumeration getPatternOccurrences()
  {
  	 return pttOccurrences.elements();
  }

  /** Returns number of pattern occurrences of this pattern
   *
   * @return number of occurrences of this pattern in different traces
   *
   */
  
  public int getNumberOfPatternOccurrences()
  {
  	 return pttOccurrences.size();
  }
}  