/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: PatternOccurrence.java								 *	
 * Package: core.dynamic										 *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.model.core.dynamic;

public class PatternOccurrence
{
  private TreeTrace trace;	// Trace that contain this pattern occurrence
  private Pattern pattern;	// Pattern that corresponds to this pattern occurrence
  private Node initiator;	// The pattern initiator
  private int freq;		    // Pattern frequency 

  /** Creates a pattern occurrence
   *
   * @param trace the trace that contains this pattern occurrence
   * @param pattern an instance of the class Pattern that describes this pattern
   * @param initiator the pattern initiator 
   * @param frequency the pattern number of repetitions
   *
   */
    
  public PatternOccurrence(TreeTrace trace, Pattern pattern, Node initiator, int frequency) 
  {
 	 this.trace = trace;
 	 this.trace.addPatternOccurrence(this);
 	 this.pattern = pattern;
 	 this.pattern.addPatternOccurrence(this);
 	 this.initiator = initiator;
 	 this.initiator.setPatternOccurrence(this);
     this.freq = frequency;
  }
  
  /** Returns the trace that contains this occurrence
   *
   * @return trace that contains this pattern occurrence
   *
   */

  public TreeTrace getTrace()
  {
  	 return trace;
  }

  /** Returns the pattern that corresponds to this pattern occurrence
   * 
   * @return the pattern that corresponds to this pattern occurrence
   *
   */

  public Pattern getPattern()
  {
  	 return pattern;
  }

  
  /** Returns the number of repetitions of the pattern
   *
   * @return pattern frequency
   *
   */

  public int getFrequency()
  {
  	return freq;
  }
  
  /** Returns the pattern initiator
   *
   * @return the pattern initiator
   *  
   */

  public Node getInitiator()
  {
  	return initiator;
  }
}  