/*
 * Created on May 6, 2004
 *
 * Node in a PartialTree. This is the wrapper class for the model node.
 * A Path is used here to denote the Node because in our specific
 * implementation, a Node can only uniquely identified by path 
 * from root node, all the intermediate parent to the current node.    
 */
package seat.ui.tree.partial;

public interface IPath {
	// create part
	public Object clone(); 						// create a copy path
	public IPath cloneUnhiddenPath();			// create till first hidden
	public IPath createChildPath(int index);	// create a child of index
	public IPath createParentPath();			// create current parent
	public void  resetSibling();				// reset sibling 
	public IPath getNextSibling();				// create next sibling
	public IPath getPrevSibling();				// create prev sibling

	// delete to node
	public boolean isHidden();					// Node is hidden?
	public boolean hasChildren();				// Node has children?
	public boolean isChildHidden(int index);	// if a child is hidden
	public int getNumberOfChildren();			// number of children
	
	// link parent/child relation
	public void reset();						// clear parent/child relation
	public void addChild(IPath child);			// add a child link
	public void setParent(IPath parent);		// set parent link
	public IPath getParent();					// get parent link
	public IPath[] getChildren();				// get all children links

	// UI display
	public String getName();					// get name of the node
	public void  setImageID(int id);			// set an image id 
	public int   getImageID();					// get image id
}
