/**
 * Core class to represent a partial tree.
 * @author lfu
 */
package seat.ui.tree.partial;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;

import seat.model.core.dynamic.Edge;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;
import seat.ui.tree.ITraceTree;
import seat.ui.tree.ITreeNode;

// root of model can always be provided by a static method in somewhere:
//ITraceLoader? so ther ewill no need to pass this para all around

public class PartialTree implements IDynamicTree,ITraceTree {
	private UniquePath pathForward;     // first path after skip n path forward
	private UniquePath pathBackward;    // first path after skip n path backward

	private HashSet expandedSet;		// contained nodes that are expanded
	private Trace trace;				// trace ctf graph
	private int sizeNeeded;				// size of number of nodes needed to be retreived
	private int totalVisibleOffest;		// offest of current visible node in tree
	private boolean keepLastTree;		// internal flag to prevent further loading
	private boolean keepPrevTree;		// internal flag to prevent further loading
	private boolean includeHidden;		// flag to consider hidden nodes when processing
	protected Hashtable lastColoredModelNode;  // some nodes in this tree are colored in prev loading  
	
	UniquePath starting;		 		// this is the main()- starting point of partial tree 
	UniquePath invisibleRoot; 			// this will be the root of UI tree to be returned
	public UniquePath top4NextRound;			// will be top tree item in the UI window
	public int lastSkippedCounter;	  		    // how many nodes are skipped in last move	
	
	
	/** 
	 * Given graph root, initialize data.
	 * @param root
	 */
	public PartialTree(Trace trace) {
		pathForward=pathBackward=null;
		expandedSet=new HashSet();
		this.trace=trace; 
		sizeNeeded=totalVisibleOffest=0;//sizeActualRetrieved=0;	
		keepLastTree=false;
		lastSkippedCounter=0;
		includeHidden=true;
		
		Edge toMain=new Edge(null,((TreeTrace)trace).getRoot());
		starting=new UniquePath(toMain);
		starting.setDescription("starting");
		starting.setTrace(trace);
		this.addToExpandSet(starting); // always open and add main method

		Edge temp=new Edge(null,((TreeTrace)trace).getRoot());			
		invisibleRoot=new UniquePath(temp); // root of UI tree to be returned, it's the same path as starting
		invisibleRoot.setDescription("Invisible");		
		top4NextRound=null;
		
		// set static var for all path-- modify to be private
		invisibleRoot.setTrace(trace);
		lastColoredModelNode=new Hashtable();
}


	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#getNextTree(partialtree.views.UniquePath, int)
	 */
	public UniquePath getNextTree(UniquePath currentNode, int offset,boolean force) {
		// TODO Auto-generated method stub
		return this.loadingForward(currentNode,sizeNeeded,offset,force);
	}

	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#getNextTree(partialtree.views.UniquePath)
	 */
	public UniquePath getNextTree(UniquePath currentNode,boolean force) {
		// TODO Auto-generated method stub
		return this.loadingForward(currentNode,sizeNeeded,1,force);
	}

	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#getPreviousTree(partialtree.views.UniquePath, int)
	 */
	public UniquePath getPreviousTree(UniquePath currentNode, int offset,boolean force) {
		// TODO Auto-generated method stub
		return this.loadingBackward(currentNode,sizeNeeded,offset,force);
	}

	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#getPreviousTree(partialtree.views.UniquePath)
	 */
	public UniquePath getPreviousTree(UniquePath currentNode,boolean force) {
		// TODO Auto-generated method stub
		return this.loadingBackward(currentNode,sizeNeeded,1,force);
	}

	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#setTreeSize(int)
	 */
	public void setTreeSize(int sizeToBeRetrieved) {
		// TODO Auto-generated method stub
		sizeNeeded=sizeToBeRetrieved;
	}

	public void setIncludeHidden(boolean flag) {
		// TODO Auto-generated method stub
		includeHidden=flag;
	}

	
	/**
	 * Using current path and epxanded set to skip to new path.
	 * Current is included in the skipped counter.
	 * The resultant path is save in pathForward. Current node 
	 * is considered as a subtree. Nodes that are opened(expanded) before
	 * are traversed recursively. Hidden nodes are not counted.
	 * @param current: staring root of the subtree
	 * @param wanted:  Number of nodes to skip 	
	 * @return; Actual number of nodes skipped.
	 */
	private int skipNodeForward(UniquePath current, int wanted) {
		int skippedCounter=1; // include the root
		pathForward=current;
		if ( (skippedCounter>=wanted) || (!isExpanded(current)) ) {
			return skippedCounter;
		}
		
		Node focused=current.getNode(),child=null;
		int length=focused.getNumberOfChildren();
		for (int i=0;i<length;i++) {
			child=focused.getChildAt(i);
			if (!includeHidden&&child.isHidden())  // hidden child is not counted 
				continue;
				
			UniquePath childPath=current.createChildPath(focused.getEdge(i));
			if (!isExpanded(childPath)) {
				skippedCounter++;
				pathForward=childPath;
				if (skippedCounter>=wanted) {
					return skippedCounter;				
				}
			} else { // recursive
				int childSkip=skipNodeForward(childPath,wanted-skippedCounter);
				skippedCounter+=childSkip;
				if (skippedCounter>=wanted) 
					return skippedCounter;
			}				
		} // loop for
	return skippedCounter;
	}
	
	/**
	 * Traverses the siblings that are after current path, retrieve wanted path.
	 * Calls processNodeForward(), so actually pointer pathForward is moved inside processNodeForward()
	 * @param current: current Path 
	 * @param wanted: number of nodes to be skipped
	 * @return: actual number found
	 */
	private int skipSiblingsForward(UniquePath current, int wanted) {
		int skippedCounter=0;
		current.resetSibling();
		UniquePath sibling=(UniquePath)current.getNextSibling();
		while (sibling!=null) {
			int sibSkip=skipNodeForward(sibling,wanted-skippedCounter);
			skippedCounter+=sibSkip;
			if (skippedCounter>=wanted) 
				return skippedCounter;
			sibling=(UniquePath)current.getNextSibling();			
		}
		return skippedCounter;
	}
	

	
	/** 
	 * Loops through all parents if there is not enough nodes.
	 * Calls processSiblingsForward()
	 * @param current: current path
	 * @param wanted: number of path to be retreived
	 * @return: actual number found 
	 */
	private int skipParentsForward(UniquePath current, int wanted) {
		int skippedCounter=0;
		UniquePath upper1Path=current.createParentPath();
		while (upper1Path!=null) { 
			skippedCounter+=skipSiblingsForward(upper1Path,wanted-skippedCounter);
			if (skippedCounter>=wanted) 
				return skippedCounter;
			upper1Path=upper1Path.createParentPath();
		} // if not enough, recursively search parents			
		return skippedCounter;
	}
				
	/**
	 * Main skip method to skip given number of nodes.
	 * Delegates calls to skipNode, skipSibling and skipParent.
	 * After skipping, the next path for loading/linking is saved in pathForward.
	 * @param current: starting path
	 * @param wanted : desired number
	 * @return: actual number skipped
	 */
	private int skipForward(UniquePath current, int wanted) {
		int skippedCounter=0;
		int toSkip=wanted+1; // +1 for the next availible position
		// skip current node first
		skippedCounter+=skipNodeForward(current,toSkip-skippedCounter);
		if (skippedCounter<toSkip)
			// skip all siblings
			skippedCounter+=skipSiblingsForward(current,toSkip-skippedCounter);
		if (skippedCounter<toSkip)
			// if still not, skip all the parents
			skippedCounter+=skipParentsForward(current,toSkip-skippedCounter);
		return skippedCounter-1;
	}
	
	/**
	 * Tests if there are enough nodes can be skipped.
	 * Delegates call to skipForward by saving/restoring pathForward
	 * @param current: starting path
	 * @param wanted : desired number
	 * @return: actual number that can be skipped
	 */
	private int testSkipForward(UniquePath current, int wanted) {
		UniquePath old=pathForward;
		int skipped=skipForward(current, wanted);
		pathForward=old;
		return skipped;
	}
	/**
	 * Tests if there is no more nodes. No pointer will be moved.
	 * @return: true- bottom
	 */
	private boolean testBottom() {
		UniquePath old=pathForward;
		int skipped=skipForward(pathForward, 1);
		pathForward=old;		
		return (skipped==0)?true:false;
	}
	
	
	/**
	 * Tries to skip certain number of nodes starting from current node.
	 * Nodes are traversed from lowest level(bottom) backwardly. Hidden nodes
	 * are not counted. When first called, 'current' needs to be prev-sibling
	 * of first UI node. 
	 * @param current: staring root of the subtree
	 * @param wanted:  Number of nodes to skip 	
	 * @return; Actual number of nodes skipped.
	 */
	private int skipNodeBackward(UniquePath current, int wanted) {
		int skippedCounter = 0;
		// contrary to forward, root(current)is included at last 
		if (isExpanded(current)) { // if expanded, go for children
			Node focused = current.getNode(), child = null;
			int length = focused.getNumberOfChildren();
			for (int i = length - 1; i >= 0; i--) {
				child = focused.getChildAt(i);
				if (!includeHidden&&child.isHidden())  // hidden child is not counted 
					continue;

				UniquePath childPath =
					current.createChildPath(focused.getEdge(i));
				if (!isExpanded(childPath)) {
					skippedCounter++;
					pathBackward = childPath;
					if (skippedCounter >= wanted) {
						return skippedCounter;
					}
				} else { // recursive
					int childSkip =
						skipNodeBackward(childPath, wanted - skippedCounter);
					skippedCounter += childSkip;
					if (skippedCounter >= wanted)
						return skippedCounter;
				}
			} // will loop if node is not enough
		}
		skippedCounter++; // current node
		pathBackward = current;
		return skippedCounter;
	}


	/**
	 * Traverse and retrieve siblings backwards.
	 * @param current: current path as a pointer
	 * @param wanted: number of nodes to be skipped
	 * @return: actual number of nodes skipped
	 */
	private int skipSiblingsBackward(UniquePath current, int wanted) {
		int skippedCounter=0;
		current.resetSibling();
		UniquePath sibling=(UniquePath)current.getPrevSibling();
		while(sibling!=null) {
			int sibSkip=skipNodeBackward(sibling,wanted-skippedCounter);
			skippedCounter+=sibSkip;
			if (skippedCounter>=wanted) 
				return skippedCounter;
			sibling=(UniquePath)current.getPrevSibling();			
		}
		return skippedCounter;
	}


	/** 
	 * Loops through all parents if there is not enough nodes.
	 * @param current: current path
	 * @param wanted: number of path to be retreived
	 * @return: actual number found 
	 */
	private int skipParentsBackward(UniquePath current, int wanted) {
		int skippedCounter=0;
		UniquePath parent=current.createParentPath(); // one by one
		while (parent!=null) {
			skippedCounter++; // this parent is counted by 1, then its siblings
			pathBackward=parent;
			if (skippedCounter>=wanted) 
				return skippedCounter;			
			skippedCounter+=skipSiblingsBackward(parent,wanted-skippedCounter);
			if (skippedCounter>=wanted) 
				return skippedCounter;
			parent=parent.createParentPath();
		} // if not enough, recursively search parents			
		return skippedCounter;
	}

	/**
	 * Main skip method to skip given number of nodes.
	 * Delegates calls to skipNode, skipSibling and skipParent.
	 * After skipping, the next path for loading/linking is saved in pathForward.
	 * @param current: starting path
	 * @param wanted : desired number
	 * @return: actual number skipped
	 */
	private int skipBackward(UniquePath current, int wanted) {
		int skippedCounter=0;
		int toSkip=wanted;
		skippedCounter+=skipSiblingsBackward(current,toSkip-skippedCounter);
		if (skippedCounter<toSkip)
			// if still not, skip all the parents
		skippedCounter+=skipParentsBackward(current,toSkip-skippedCounter);
		return skippedCounter;
	}
	
	/**
	 * Tests if there are enough nodes can be skipped.
	 * @param current: starting path
	 * @param wanted : desired number
	 * @return: actual number that can be skipped
	 */
	private int testSkipBackward(UniquePath current, int wanted) {
		UniquePath old=pathBackward;
		int skipped=skipBackward(current, wanted);
		pathBackward=old;
		return skipped;
	}
	/**
	 * Tests if top of tree has beem reached.
	 * @return: true for top
	 */
	private boolean testTop() {
		UniquePath old=pathBackward;
		int skipped=skipBackward(pathBackward, 1);
		pathBackward=old;
		return (skipped==0)?true:false;
	}


	/**
	 * Link the a node/subtree. Relations between parent/current will be created.
	 * If current is expanded before, it will be treated as a subtree, and all 
	 * the children will be created and linked recursively.
	 * Note: Before calling this method, parent/child SHOULD be not linked. Otherwise
	 * it will be re-linked which case is error. 
	 * @param parent: parent node of current.
	 * @param current: current node to be processed
	 * @param wanted: maximum number of nodes desiried to be linked from current and its children 
	 * @return: actual number linked.
	 */
	private int linkNodeForward(UniquePath parent,UniquePath current, int wanted) {
		int skippedCounter=1; // include the root
		pathForward=current;
		parent.addChild(current);
		current.setParent(parent);
		if ( (skippedCounter>=wanted) || (!isExpanded(current)) ) {
			return skippedCounter;
		}
		
		Node focused=current.getNode(),child=null;
		int length=focused.getNumberOfChildren();
		for (int i=0;i<length;i++) {
			child=focused.getChildAt(i);
			if (!includeHidden&&child.isHidden())  // hidden child is not counted 
				continue;
				
			UniquePath childPath=current.createChildPath(focused.getEdge(i));
			childPath.setTrace(trace);
			if (!isExpanded(childPath)) {
				current.addChild(childPath);  // link node
				childPath.setParent(current);
				skippedCounter++;
				pathForward=childPath;
				if (skippedCounter>=wanted) {
					return skippedCounter;				
				}
			} else { // recursive
				int childSkip=linkNodeForward(current,childPath,wanted-skippedCounter); // link is done in call
				skippedCounter+=childSkip;
				if (skippedCounter>=wanted) 
					return skippedCounter;
			}				
		} // loop for
	return skippedCounter;
	}
	
	/**
	 * Link siblings of current node
	 * @param parent: parent path to be set in the parentUI field of current path
	 * @param current: current Path 
	 * @param wanted: number of nodes to be retrieved
	 * @return: actual number found
	 */
	private int linkSiblingsForward(UniquePath parent,UniquePath current, int wanted) {
		int skippedCounter=0;
		UniquePath sibling=(UniquePath)current.getNextSibling();
		while(sibling!=null) {
			int sibSkip=linkNodeForward(parent,sibling,wanted-skippedCounter);
			skippedCounter+=sibSkip;
			if (skippedCounter>=wanted) 
				return skippedCounter;
			sibling=(UniquePath)current.getNextSibling();
		}
		return skippedCounter;
	}

	/** 
	 * Loops through all parents if there is not enough nodes.
	 * There is NO parent parameter as in previous node/siblng linking method, 
	 * because parent field can always be found thru getParent(). 
	 * @param current: current path
	 * @param wanted: number of path to be retreived
	 * @return: actual number found 
	 */
	private int linkParentsForward(UniquePath current, int wanted) {
		int skippedCounter=0;
		UniquePath upper1Path=(UniquePath)current.getParent();
		UniquePath upperParent=(UniquePath)upper1Path.getParent();			
		while ((upper1Path!=null)&&(upperParent!=null)) { // when reach inv, link will terminate
			skippedCounter+=linkSiblingsForward(upperParent,upper1Path,wanted-skippedCounter);
			if (skippedCounter>=wanted) 
				return skippedCounter;
			upper1Path=(UniquePath)upper1Path.getParent();
			upperParent=(UniquePath)upper1Path.getParent();
		} // if not enough, recursively search parents
		return skippedCounter;
	}

	

	/** Link is always done in forward, no backward
	 * Main Method for link, called other linking methods.
	 * @param parent
	 * @param child
	 * @param wanted
	 * @return
	 */
	private int linkForward(UniquePath parent,UniquePath child, int wanted) {
		int desiredCounter=0;
		desiredCounter+=linkNodeForward(parent,child,wanted-desiredCounter);
		if (desiredCounter>=wanted)
			return desiredCounter;
		desiredCounter+=linkSiblingsForward(parent,child,wanted-desiredCounter);
		if (desiredCounter>=wanted) 
			return desiredCounter;
		desiredCounter+=linkParentsForward(child,wanted-desiredCounter);
		return desiredCounter;				
	}

	/**
	 * Main loading method, called by public interface method.
	 * Two steps first skip number of nodes(forth and back if no enough number),
	 * then load desired number of nodes. 
	 * @param current: starting pointing to for loading
	 * @param wanted:  number of nodes to be loaded into the partial tree
	 * @param skipped: number of nodes to be skipped
	 * @param forceRefresh: if true, be sure to relaod tree, otherwise, may keep current for efficiency.
	 * @return:        root of the partial tree after loading
	 */				
	// Clarify ideas: we are creating a partial tree, and reference to the old one.
	// Thereofore, most UniquePaths belongs to new tree. siblings need to be reset when used 
	// determine top4NextRound display is important 
	private UniquePath loadingForward(UniquePath current, int wanted,int skipped,boolean forceRefresh) {
		if (testBottom()&&!forceRefresh) // reach bottom
			return invisibleRoot;

		// step1. try to see if all nodes in path are visible
		Node[] nodes=current.getAllNodes();
		if (!includeHidden&&nodes[0].isHidden()) { // root is hidden
			starting.reset(); //clear children
			return invisibleRoot;
		}
		
		UniquePath lastVisible;
		if (includeHidden)
			lastVisible=current;
		else { 
			int length=0; // length of edges that is not hidden
			for (int i=0;i<nodes.length;i++) {
				if (!nodes[i].isHidden()) 
					length++;
				else
					break;
			}		
			lastVisible=new UniquePath(current,length); // check if all node are visble
		}

		// Step 2. only skip where all nodes along the path are visible
		UniquePath realSkipPoint;
		int realWant,realSkip;
		if (lastVisible.equals(current)) { // only skip when no hidden nodes			
			int total=wanted+skipped;
			int test=testSkipForward(lastVisible,total);
			if (test>=wanted) { // enough for last page
				realSkipPoint=lastVisible;
				realWant=wanted;
				realSkip=test-wanted;		
			} else {
				int backNeeded=wanted-test;
				int back=skipBackward(lastVisible,backNeeded); // real move
				realSkipPoint=pathBackward;
				realWant=test+back;
				realSkip=0;
			}					
		} else { // no skip
			int total=wanted;
			int test=testSkipForward(lastVisible,total);
			if (test>=wanted) { // enough for last page
				realSkipPoint=lastVisible;
				realWant=wanted;
				realSkip=0;		
			} else {
				int backNeeded=wanted-test;
				int back=skipBackward(lastVisible,backNeeded); // real move
				realSkipPoint=pathBackward;
				realWant=test+back;
				realSkip=0;
			}								
		}
		
		lastSkippedCounter=skipForward(realSkipPoint,realSkip);
		if (realSkip>0) { // from new point
			top4NextRound=pathForward;
		} else { // use realLoadPoint since no skip
			top4NextRound=realSkipPoint;
		}
		pathBackward=top4NextRound; // adjust pointer for backward

		starting.reset();
		Edge[] ns1=top4NextRound.getAllEdges();
		UniquePath parent=starting,child=null;
		for (int i=0;i<ns1.length;i++) {
			child=parent.createChildPath(ns1[i]);
			if (i!=ns1.length-1) {// last one is not lined here, link is done universally in linkNodeForward
				parent.addChild(child);  // link all other nodes except last one
				child.setParent(parent);
				parent=child;
			}
		}
		if (child==null) {// if pathForward is main, will skip above for
			parent=invisibleRoot;
			child=starting;
			parent.reset();
			child.reset();
		}
		
		linkForward(parent,child,realWant+1); // plus one for current
		return invisibleRoot;
	}

	/** 
	 * Loads children from a previous point.
	 * @param current
	 * @param wanted
	 * @param skipped
	 * @param forceRefresh
	 * @return
	 */
	private UniquePath loadingBackward(UniquePath current, int wanted,int skipped,boolean forceRefresh) {
		if (testTop()&&!forceRefresh) // reach bottom
			return invisibleRoot;

		// step1. try to see if all nodes in path are visible
		Node[] nodes=current.getAllNodes();
		if (!includeHidden&&nodes[0].isHidden()) { // root is hidden
			starting.reset(); //clear children
			return invisibleRoot;
		}

		UniquePath lastVisible;
		if (includeHidden)
			lastVisible=current;
		else { 
			int length=0; // length of edges that is not hidden
			for (int i=0;i<nodes.length;i++) {
				if (!nodes[i].isHidden()) 
					length++;
				else
					break;
			}		
			lastVisible=new UniquePath(current,length); // check if all node are visble
		}
		
		// Step 2. only skip where all nodes along the path are visible
		UniquePath realSkipPoint;
		int realWant,realSkip;
		if (lastVisible.equals(current)) { // only skip when no hidden nodes			
			realSkip=skipBackward(lastVisible,skipped);
			realSkipPoint=pathBackward;
			realWant=testSkipForward(realSkipPoint,wanted);			
		} else { // no skip
			realSkip=0;		
			realSkipPoint=lastVisible;
			realWant=testSkipForward(realSkipPoint,wanted);
		}
		
		top4NextRound=realSkipPoint;
		lastSkippedCounter=realSkip;
		
		starting.reset();
		Edge[] ns1=top4NextRound.getAllEdges();
		UniquePath parent=starting,child=null;
		for (int i=0;i<ns1.length;i++) {
			child=parent.createChildPath(ns1[i]);
			if (i!=ns1.length-1) {// last one is not lined here, link is done universally in linkNodeForward
				parent.addChild(child);  // link all other nodes except last one
				child.setParent(parent);
				parent=child;
			}
		}
		if (child==null) {// if pathForward is main, will skip above for
			parent=invisibleRoot;
			child=starting;
			parent.reset();
			child.reset();
		}
		
		linkForward(parent,child,realWant+1); // plus one for current
		return invisibleRoot;
	}		
		
	/**
	 * Loads initial data when the system starts.
	 * @return: root
	 */			
	public UniquePath loadingStartup() {
		// clear relation first, they will be setup universally by link
		invisibleRoot.reset();
		starting.reset();
		top4NextRound=pathForward=pathBackward=starting;
		loadingForward(starting, this.sizeNeeded,0, true);
		return invisibleRoot;		
	}
	
	/**
	 * Adds the apth to the expanded set
	 * @param path
	 */
	public void addToExpandSet(UniquePath path) {
		// use HashSet for list
		if (!isExpanded(path))
			expandedSet.add(path);
	}
	
	/**
	 * Removes the path from the expanded set
	 * @param path
	 */
	public void removeFromExpandSet(UniquePath path) {
		UniquePath found=null;
		Iterator it=expandedSet.iterator();
		while (it.hasNext()) {
			UniquePath next=(UniquePath)it.next();
			if (next.equals(path)) {
				found=next;
				break;
			}
		}
		if (found!=null)
			expandedSet.remove(found);
	}
	
	/**
	 * Tests if the path is opened.
	 * @param path
	 * @return: true if it is expanded
	 */
	public boolean isExpanded(UniquePath path) {
		Iterator it=expandedSet.iterator();
		while (it.hasNext()) {
			UniquePath next=(UniquePath)it.next();
			if (next.equals(path))
				return true;
		}		
		return false;
	}		
	
	/** How many nodes are there in the partial tree
	 *  @return: tree size
	 */
	public int calcTreeSize() {
		Iterator it=expandedSet.iterator();
		Node expanded;
		int counter=1;// +1 one for main	
		while (it.hasNext()) {
			Node expandedNode=((UniquePath)it.next()).getNode();
			int length=expandedNode.getNumberOfChildren();
			for (int i=0;i<length;i++)
				if (!expandedNode.getChildAt(i).isHidden())
					counter++;
		}
		return counter; 	
	}
	
	
	/**
	 * Returns offset of current path, so it can be sued to set Scrollbar position  
	 */
	public int calcFocusOffset() {
		return this.totalVisibleOffest;
	}
	

	// helper method to display the tree found
	public void printing(UniquePath path)
	{
	   restoreRepHelper(path,0);
	   Iterator it=expandedSet.iterator();
	   System.out.println("--------Expandedlist-----");	   	
	   while (it.hasNext()) {
			UniquePath up=(UniquePath)it.next();
			System.out.println(" " +up.getName());	   	
	   }
	   System.out.println("--------end of list-----");	   	
	}
  
	// called by printing
	private void restoreRepHelper(UniquePath path,int level)
	{
		if (path==null)
			return;
		for (int i=0;i<level;i++)
			System.out.print("  ");
	 	System.out.println(path.getName());
	 	UniquePath[] children=(UniquePath[])path.getChildren();
	 	for (int i=0;i<children.length;i++) {
			restoreRepHelper(children[i],level+1);
	 	}
  }

	/* (non-Javadoc)
	 * @see seat.ui.tree.ITraceTree#construct()
	 */
	public ITreeNode construct() {
		// TODO Auto-generated method stub
		return getStarting();
	}

	/* (non-Javadoc)
	 * @see seat.ui.tree.ITraceTree#getStarting()
	 */
	public ITreeNode getStarting() {
		// TODO Auto-generated method stub
		return starting;
	}
	/* (non-Javadoc)
	 * @see seat.ui.tree.ITraceTree#getInvisibleRoot()
	 */
	public ITreeNode getInvisibleRoot() {
		// TODO Auto-generated method stub
		return invisibleRoot;
	}
	
	/**
	 * @return
	 */
	public Hashtable getLastColoredModelNode() {
		return lastColoredModelNode;
	}

	/**
	 * @param node
	 */
	public void setLastColoredModelNode(Node node,int color) {
		lastColoredModelNode.clear();  
		if (node==null)	return;
		lastColoredModelNode.put(node,new Integer(color));
//		if (node.getNumberOfChildren()<=0)
//			return;
//		Enumeration en=node.getChildren();
//		while (en.hasMoreElements()) {
//			lastColoredModelNode.put(en.nextElement(),new Integer(color));			
//		}
	}
}
