/*
 * Created on Apr 23, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.ui.tree.partial; 

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ScrollBar;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import seat.SeatUtil;
import seat.model.core.dynamic.Trace;
import seat.ui.tree.ITraceTree;
import seat.ui.tree.ITreeNode;
import seat.ui.tree.TraceViewer;

/**
 * @author lfu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class PartialTreeViewer extends TraceViewer {
	private Composite drawArea;
	private Tree treeUI;			// Tree control widget
	private ScrollBar vertical;     // vertical scroll bar of tree widget

	/**
	 * UI refresh is ready, but height calc is ready yet.
	 * So need move tree loading to createPartControl of parent.
	 * @param parent: parent control
	 * @param root: root of graph
	 */
	public PartialTreeViewer(Composite parent, Trace trace) {
		super(parent,trace, SWT.NULL); 
		// TODO Auto-generated constructor stub
		drawArea=parent;
		this.treeUI=(Tree)this.getControl();
		this.vertical=(ScrollBar)treeUI.getVerticalBar();
		
		this.addTreeListener(new TreeExpandListener(this));		
		vertical.addSelectionListener(new TreeScrollListener(this));	
		this.addDoubleClickListener(new TreeDoubleClickListener(this));

		initialLoading();
		
	}

	public ITraceTree constructFromTrace(Trace trace) {
		// TODO Auto-generated method stub		
		PartialTree ct=new PartialTree(trace);
		ct.construct();
		return ct;
	}

	// To calculate window size, this method must be called after window creation.
	// Do NOT called this method in PartialTreeViewer constructor, 
	// or ViewPart createPartControl(), init()
	private void initialLoading() {
		((PartialTree)tree).setTreeSize(10);
		((PartialTree)tree).loadingStartup();
//		this.partialTree.printing(partialTree.invisibleRoot);  // for debugging
		treeUI.setRedraw(false);
		this.setInput(((PartialTree)tree).invisibleRoot);
		this.expandToLevel(2);
		treeUI.setRedraw(true);	
		this.refresh();
	}

	/**
	 * Refresh UI window with new partial tree 
	 * @param newRoot
	 */
	public void refreshViewer(ITreeNode node) {
		if (treeUI==null) return;
		setImages();
		treeUI.setRedraw(false);
		this.internalRefresh(((PartialTree)tree).starting);
		this.expandAll();
		treeUI.setRedraw(true);	
		setFirst(((PartialTree)tree).top4NextRound);
		refreshPlus();// put behind this.refresh(); or remove this.refresh();
		setScrollBar();  // must not be put before redraw
	}


	public void refreshViewerWithReload() {
		String filter=SeatUtil.getStringSetting(SeatUtil.TRACE_SHOWFILTERED);
		boolean includeFiltered=Boolean.valueOf(filter).booleanValue();
		lateRefreshWithReload2(includeFiltered);
	}

	public void lateRefreshWithReload2(boolean includeFiltered) {
		setImages();
		UniquePath top=((PartialTree)tree).top4NextRound;
		((PartialTree)tree).setIncludeHidden(includeFiltered);
		UniquePath partialRoot=((PartialTree)tree).getNextTree(top,0,true);
		refreshViewer(((PartialTree)tree).invisibleRoot);
	}

	/**
	 * Calculate how many nodes can be dispalyed in current view widows.
	 * @return: number of nodes
	 */
	public int calcWindowCapacity() {
		int clientHi=drawArea.getClientArea().height;
		int cap=clientHi/treeUI.getItemHeight();
		((PartialTree)tree).setTreeSize(cap);
		return cap;
	}

	private void setScrollBar() {
		int treeSize=((PartialTree)tree).calcTreeSize();
		int windSize=calcWindowCapacity();
		if (treeSize>windSize) {  // show vertical scroll bar
			vertical.setValues(((PartialTree)tree).lastSkippedCounter,0,treeSize,windSize,1,10);
			vertical.setVisible(true);
//System.out.println("Scroll:" +"(Now)-" +((PartialTree)tree).lastSkippedCounter+ "(Size)-" +treeSize);
		}
		else 
			vertical.setVisible(false);
	}

	private void refreshPlus() {
		Tree wt=(Tree)this.getControl();
		TreeItem[] mains=wt.getItems(); // multithread/mains
		for (int i=0;i<mains.length;i++) 
			refreshOnePlus(mains[i]);
	}
			
	private void refreshOnePlus(TreeItem item) {
		if ((item==null)||(item.getData()==null))
			return;
		updatePlus(item,((UniquePath)item.getData()));
		TreeItem[] children=item.getItems();
		for (int i=0;i<children.length;i++) {
			refreshOnePlus(children[i]); 
		}
	}

	private void setFirst(UniquePath first) {
		System.out.println("*****First Node in Window: "+ first.getName());

		Tree wt=(Tree)this.getControl();
		TreeItem[] mains=wt.getItems(); // multithread/mains
		TreeItem top;
		for (int i=0;i<mains.length;i++) {
			top=findFirstItem(mains[i],first);			
			if (top!=null) {
//				OS.SendMessage (wt.handle, OS.TVM_SELECTITEM, OS.TVGN_FIRSTVISIBLE, top.handle);
				wt.setTopItem(top);
				return;
			}
		}
	}
	
	// the first path(node) to be dispalyed will be Forward in partial tree obj
	private TreeItem findFirstItem(TreeItem item,UniquePath first) {
		if (first.equals((UniquePath)item.getData())) 				
			return item;
		TreeItem[] items=item.getItems();
		TreeItem found=null;
		for (int i=0;i<items.length;i++) {
			if (first.equals((UniquePath)items[i].getData())) 				
				return items[i];
			else {
				found=findFirstItem(items[i],first);
				if (found!=null)
					return found;
			}
		}
		return null;					
	}
	
	
	/**
	 * @return
	 */
	public Composite getDrawArea() {
		return drawArea;
	}

	/**
	 * @return
	 */
	public ScrollBar getVertical() {
		return vertical;
	}
	
}
