/*
 * Created on May 7, 2004
 * 
 * When a node in the tree is double clicked, it will jump to 
 * the method location in source code.
 *  
 */
package seat.ui.tree.partial;

import java.util.Hashtable;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IParent;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.ui.IWorkingCopyManager;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;

import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.ui.algorithm.JavaParsing;
import seat.ui.algorithm.NameParsing;
import seat.ui.tree.ITraceViewer;
import seat.ui.tree.ITreeNode;

public class TreeDoubleClickListener extends TreeListener implements IDoubleClickListener {

	/**
	 * 
	 */
	public TreeDoubleClickListener(ITraceViewer treeViewer) {
		super(treeViewer);
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IDoubleClickListener#doubleClick(org.eclipse.jface.viewers.DoubleClickEvent)
	 */
	public void doubleClick(DoubleClickEvent event) { // jump to src
		// TODO Auto-generated method stub
		ISelection select=event.getSelection();
		Object obj = ((IStructuredSelection)select).getFirstElement();	
		if (obj instanceof ITreeNode) {
			Node node=((ITreeNode)obj).getElement();
			String pckClassName=NameParsing.getLongClassName(node);
			String methodName=NameParsing.getShortMethodName(node);

			// the following code converts <init> to constructor-class name
			if (methodName.equalsIgnoreCase("<init>"))
				methodName=NameParsing.getShortClassName(node);

			Hashtable classCache=treeViewer.getTrace().getState().getJavaClasses();
			IParent topLevel=(IParent)classCache.get(pckClassName);
			if (topLevel==null) {
				// find it			
				IProject prj=treeViewer.getActiveProject();
				IWorkspaceRoot root=ResourcesPlugin.getWorkspace().getRoot();
				IResource project=root.findMember(prj.getFullPath());
				IFile file=((IContainer)project).getFile(new Path(pckClassName+".java"));
				
				try {
					// must add requires org.eclipse.jdt.core, ui in pluhin.xml to avoid errors 
					ICompilationUnit unit=JavaCore.createCompilationUnitFrom(file);
					IEditorPart javaEditor=JavaUI.openInEditor(unit);
					IWorkingCopyManager manager= JavaPlugin.getDefault().getWorkingCopyManager();
					IJavaElement fInput=manager.getWorkingCopy(javaEditor.getEditorInput());
				
					if (fInput instanceof IParent) {
						IParent classFile= (IParent) fInput;
						IJavaElement[] javaElements=classFile.getChildren();
						for (int j=0;j<javaElements.length;j++)
							if (javaElements[j].getElementName().equals(NameParsing.getShortClassName(node))) {
								topLevel= (IParent) javaElements[j];
								break;
							}
					}
				}catch(JavaModelException jme) {}
				catch (PartInitException pie) {}	
			}
			
			if (topLevel==null)
				return;
				
			try {
				IJavaElement[] children=topLevel.getChildren();
				boolean found=false;
				for (int i=0;i<children.length;i++) {
					if (children[i].getElementName().equals(methodName) &&(children[i].getElementType()==IJavaElement.METHOD) ) {
						found=true;
						IMethod method=(IMethod)children[i]; 
						IEditorPart javaEditor=JavaUI.openInEditor((IJavaElement)method);
						JavaUI.revealInEditor(javaEditor,(IJavaElement)method);
						break;
					}
				}
				if (!found) {
					SeatUtil.showMessage("Source code cannot be found!");
					if (topLevel instanceof IJavaElement) {
						IEditorPart javaEditor=JavaUI.openInEditor((IJavaElement)topLevel);
						JavaUI.revealInEditor(javaEditor,(IJavaElement)topLevel);						
					}	
				}
			} catch(JavaModelException jme) {}
			catch (PartInitException pie) {}	
		}
	}	 
	 
	public void doubleClick2(DoubleClickEvent event) {
		ISelection select=event.getSelection();
		Object obj = ((IStructuredSelection)select).getFirstElement();	
		if (obj instanceof ITreeNode) {
			Node node=((ITreeNode)obj).getElement();
			String qualifier=node.getLabel();
			int index=qualifier.lastIndexOf('.');
			String methodName=(index==-1)?qualifier:qualifier.substring(index+1);
			String pc=(index==-1)?"":qualifier.substring(0,index);
			index=pc.lastIndexOf('.');			
			String packageName=(index==-1)?"":pc.substring(0,index);
			String className=(index==-1)?"":pc.substring(index+1);
			packageName=packageName.replace('.','/');
			
			// find it			
			IProject prj=treeViewer.getActiveProject();
			IWorkspaceRoot root=ResourcesPlugin.getWorkspace().getRoot();
			IResource project=root.findMember(prj.getFullPath());
			Path p=new Path(packageName+"/"+className);
			p=(Path)p.addFileExtension("java");
			IFile file=((IContainer)project).getFile(p);//new Path(packageName+"/"+className+".java"));
			
			Trace trace=((ITreeNode)obj).getTrace();
			JavaParsing jp=new JavaParsing(trace);
			jp.parse();
		}			
	}
	public void doubleClick33(DoubleClickEvent event) { // jump to src
		// TODO Auto-generated method stub
		ISelection select=event.getSelection();
		Object obj = ((IStructuredSelection)select).getFirstElement();	
		if (obj instanceof ITreeNode) {
			Node node=((ITreeNode)obj).getElement();
			String qualifier=node.getLabel();
			int index=qualifier.lastIndexOf('.');
			String methodName=(index==-1)?qualifier:qualifier.substring(index+1);
			String pc=(index==-1)?"":qualifier.substring(0,index);
			index=pc.lastIndexOf('.');			
			String packageName=(index==-1)?"":pc.substring(0,index);
			String className=(index==-1)?"":pc.substring(index+1);
	    	packageName=packageName.replace('.','/');
			
			// find it			
			IProject prj=treeViewer.getActiveProject();
			IWorkspaceRoot root=ResourcesPlugin.getWorkspace().getRoot();
			IResource project=root.findMember(prj.getFullPath());
			IFile file=((IContainer)project).getFile(new Path(packageName+"/"+className+".java"));
			
			// must add requires org.eclipse.jdt.core, ui in pluhin.xml to avoid errors 
			//JavaCore java=JavaCore.getJavaCore();
			try {
				ICompilationUnit unit=JavaCore.createCompilationUnitFrom(file);
				IEditorPart javaEditor=JavaUI.openInEditor(unit);

				IWorkingCopyManager manager= JavaPlugin.getDefault().getWorkingCopyManager();
				IJavaElement fInput=manager.getWorkingCopy(javaEditor.getEditorInput());
				
				if (fInput instanceof IParent) {
					IParent classFile= (IParent) fInput;
					IJavaElement[] javaElements=classFile.getChildren();
					for (int j=0;j<javaElements.length;j++)
						if (javaElements[j].getElementName().equals(className)) {
							IParent topLevel= (IParent) javaElements[j];
							IJavaElement[] children=topLevel.getChildren();
							for (int i=0;i<children.length;i++) {
								if (children[i].getElementName().equals(methodName) &&(children[i].getElementType()==IJavaElement.METHOD) ) {
									IMethod method=(IMethod)children[i]; 
									if (javaEditor!=null)
										JavaUI.revealInEditor(javaEditor, (IJavaElement)method);
									break;
								}
						}
								
					}
				}
									
//				IType type=unit.getType(methodName);
//				IMethod method=type.getMethod(methodName,null);
//				if (method!=null) {
//					if (javaEditor!=null)
//							JavaUI.revealInEditor(javaEditor, (IJavaElement)method);
//				}
			} catch(JavaModelException jme) {}
			catch (PartInitException pie) {}	
		}
	}
	
	protected IType getMainType(ICompilationUnit compilationUnit) {
		String name= compilationUnit.getElementName();
		int index= name.indexOf('.');
		if (index != -1)
			name= name.substring(0, index);
		IType type= compilationUnit.getType(name);
		return type.exists() ? type : null;
	}

}
