package seat.ui.tree.partial;

import org.eclipse.jface.viewers.ITreeViewerListener;
import org.eclipse.jface.viewers.TreeExpansionEvent;

import seat.ui.tree.ITraceViewer;

/**
 * A listener for the tree expanding event. It will retrieve the subnodes
 * of the expanded nodes, refresh the tree display
 * @author lfu
 * @see org.eclipse.jface.viewers.ITreeViewerListener.
 */
public class TreeExpandListener extends TreeListener implements ITreeViewerListener {
	/**
	 * Constructor. 
	 * @param treeViewer that need to be listened.
	 */
	public TreeExpandListener(ITraceViewer treeViewer) {
		super(treeViewer);
	}

	/** Removes collapsed item from list in PartialTree, and refresh UI tree
	 * @see org.eclipse.jface.viewers.ITreeViewerListener#treeCollapsed(org.eclipse.jface.viewers.TreeExpansionEvent)
	 */
	public void treeCollapsed(TreeExpansionEvent event) {
		UniquePath selected=(UniquePath)event.getElement();
		((PartialTree)partialTree).removeFromExpandSet(selected);
		UniquePath top=((PartialTree)partialTree).top4NextRound;
		UniquePath partialRoot=((PartialTree)partialTree).getNextTree(top,0,true);
		treeViewer.refreshViewer(partialRoot);
	}
	
	/**
	 * Adds expanded item to PartialTree expanded list;
	 * Retrieve subnodes of this selected node and refreshes UI tree.
	 * @see org.eclipse.jface.viewers.ITreeViewerListener#treeExpanded(org.eclipse.jface.viewers.TreeExpansionEvent)
	 */
	public void treeExpanded(TreeExpansionEvent event) {
		UniquePath selected=(UniquePath)event.getElement();
		((PartialTree)partialTree).addToExpandSet(selected);
	    UniquePath top=((PartialTree)partialTree).top4NextRound;
		UniquePath partialRoot=((PartialTree)partialTree).getNextTree(top,0,true);
		treeViewer.refreshViewer(partialRoot);
	}		
}
