/**
 * Created on Mar 12, 2004
 * @author lfu
 * This class belongs to model part representation. Tt reprents a path
 * from root model node to current node. This rational of existence of 
 * this class is that a model Node can NOT unqiuely identfied within graph
 * becasue a Node can have several parents.  
 */
package seat.ui.tree.partial;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.ui.views.properties.IPropertySource;

import seat.model.core.dynamic.Edge;
import seat.model.core.dynamic.Node;
import seat.ui.tree.ITreeNode;
import seat.ui.tree.TreeNode;
import seat.ui.views.MethodPropertySource;

public class UniquePath extends TreeNode implements IAdaptable {
	private static UniquePath[] EMPTY_ARRAY = new UniquePath[0];
	private ArrayList edges=new ArrayList(); // containes edges of a path
	private int nextSiblingIndex=-999; 
	private int prevSiblingIndex=-999; 
//	private static Trace trace;
	/**
	 * Construct the path from another path;
	 * When null is given as node, an empty list is inited internally
	 * @see UniquePath(UniquePath another,int length)  
	 */
	public UniquePath(UniquePath node) {
		super();  // set null first, then use setNode();
		if (node==null)
			return;
		for (int i=0;i<node.edges.size();i++)
			edges.add(node.edges.get(i));
		setElement(this.getNode());
	}
	/**
	 * Create a path from another one, this created path is a trunct part
	 * @param another: original path
	 * @param length:  how many edges to be included in the new path
	 */
	public UniquePath(UniquePath another,int length) {
		super();
		int len;
		if ((length>another.edges.size()||length<0))
			len=another.edges.size();
		else
			len=length;
		for (int i=0;i<len;i++)
			edges.add(another.edges.get(i));
		setElement(this.getNode());
	}

	/**
	 * This is only for startup initialization.
	 * Edge will be virtual edge to main method
	 * @param edge: first edge in the tree
	 */
	public UniquePath(Edge edge) {
		super();
		edges.add(edge); 
		setElement(this.getNode());
	}

	/**
	 * used internally to create a path from a list of edges
	 * @param newEdges: edge array.
	 */
	private UniquePath(ArrayList newEdges) {
		super();
		for (int i=0;i<newEdges.size();i++)
			edges.add(newEdges.get(i)); 
		setElement(this.getNode());
	}
	
	/**
	 * Returns the Node in the last edge; null if not exist
	 * @return
	 */
	public Node getNode() {
		if (edges.size()>0) {
			Edge last=(Edge)(edges.get(edges.size()-1));
			return (last!=null)?last.getSecondNode():null;				
		}
		else
			return null; 
	}
	
	/**
	 * Returns all the Node start from the root along the path/edges,
	 * including the last one. Status of node(hidden/show) is disregarded.
	 * @return: Node list or empty array
	 */
	// Note: this is method is only used to test hidden.
	// Other calls are replaced by getAllEdges and no longer called. 
	public Node[] getAllNodes() {
		ArrayList nodes=new ArrayList();
		// becauae a virtual node for main is used, use getSecondNode
		for (int i=0;i<edges.size();i++) {
			Edge edge=(Edge)edges.get(i);
			if (edge.getFirstNode()!=null)
				nodes.add(edge.getFirstNode());
		}
		Node last=getNode();
		if (last!=null)
			nodes.add(last); 
		
		return (Node[])nodes.toArray(new Node[nodes.size()]);
	}	
	
	/**
	 * Returns all the Edges a path contains. The returned set does NOT
	 * contain the temp edge from invisibleRoot to main.  
	 * 
	 * @return: Edge list starting from main
	 */
	public Edge[] getAllEdges() {
		 ArrayList med=new ArrayList();
		 for (int i=1;i<edges.size();i++) { // start from 1, not include first virtuil one
		 	med.add(edges.get(i));
		 }
		 return (Edge[])med.toArray(new Edge[med.size()]);
	}
	/**
	 * Returns parent path, which cut the current path short.
	 * Parent path can be NULL. Called private constructor.
	 * Note: This method will create a new path from current path.
	 */
	public UniquePath createParentPath() {
		int shortLength=edges.size()-1;
		if (shortLength<=0) // no parent
			return null;
		ArrayList shortenedEdges=new ArrayList();
		for (int i=0;i<shortLength;i++)
			shortenedEdges.add(edges.get(i));
		return new UniquePath(shortenedEdges); 
	}

	/**
	 * Creates a new child path from parent path and the give child node.
	 * if child node is not actually the child of parent, only parent path
	 * will be returned. 
	 * @param child:child node in path
	 * @return: new child path, or path same as parent
	 */
	// Bug correction: can NOT use the child node as the para, because two same child
	// can be same, but with diffrent in-Edge
	public UniquePath createChildPath(Edge childEdge) {
		ArrayList childList=new ArrayList();
		for (int i=0;i<edges.size();i++)
			childList.add(edges.get(i));
		if (childEdge!=null)
			childList.add(childEdge);
		return new UniquePath(childList); 
	}

	/**
	 * Sets Sibling index to initial state, so new search can begin again
	 */
	public void resetSibling() {
		nextSiblingIndex=-999; 
		prevSiblingIndex=-999; 		
	}
	/**
	 * When called every time ,will return the next availible sibling node; or null if no more
	 */
	public ITreeNode getPrevSibling() {
		int shortLength=edges.size()-1;
		if (shortLength<=0) // no parent
			return null;		
		Edge current=(Edge)(edges.get(shortLength));
		Node parentNode=current.getFirstNode();
		if (parentNode==null)
			return null;
		if (this.prevSiblingIndex==-999) { // not searched yet
			int edgeIndex=-1;
			int childCnt=parentNode.getNumberOfChildren();
			for (int i=0;i<childCnt;i++)
				if (parentNode.getEdge(i)==current) {
					edgeIndex=i;
					break;
				}
			if (edgeIndex<=0) // 0 is first one, no previous sibling
				return null;
			this.prevSiblingIndex=edgeIndex-1; // first availible sib index
		} else if (this.prevSiblingIndex==-1) // has already bypass first one
			return null;
			
		// there is a sibling found, and test if it's hidden
		Edge sibling=parentNode.getEdge(this.prevSiblingIndex);
		this.prevSiblingIndex--;		
		ArrayList shortenedEdges=new ArrayList();
		for (int i=0;i<shortLength;i++)
			shortenedEdges.add(edges.get(i));
		shortenedEdges.add(sibling);
		return new UniquePath(shortenedEdges); 
	}
	
	// methos to get one next siblign at a time
	public ITreeNode getNextSibling() {
		int shortLength=edges.size()-1; // parent edge size
		if (shortLength<=0) // no parent
			return null;		
		Edge current=(Edge)(edges.get(shortLength));
		Node parentNode=current.getFirstNode();
		if (parentNode==null)
			return null;
		if (this.nextSiblingIndex==-999) { // not searched yet
			int edgeIndex=-1;
			int childCnt=parentNode.getNumberOfChildren();
			for (int i=0;i<childCnt;i++)
				if (parentNode.getEdge(i)==current) {
					edgeIndex=i;
					break;
				}
			if ((edgeIndex==-1)||(edgeIndex>=childCnt-1) ) // cuurent idx is last one ,no pre
				return null;
			this.nextSiblingIndex=edgeIndex+1; // first availible next index
		} else if (this.nextSiblingIndex>=parentNode.getNumberOfChildren()) // has already bypass first one
			return null;
				
		// there is a sibling found, and test if it's hidden, do not use node to find edge, use index
		Edge sibling=parentNode.getEdge(this.nextSiblingIndex);
		this.nextSiblingIndex++;		
		ArrayList shortenedEdges=new ArrayList();
		for (int i=0;i<shortLength;i++)
			shortenedEdges.add(edges.get(i));
		shortenedEdges.add(sibling);
		return new UniquePath(shortenedEdges); 
	}
	
	/** Decides how to consider two path are same
	 * Criteria: Though two paths can be diffrent obj, if they edges are same,
	 * 	they will be considered same.
	 */
	public boolean equals(Object o) {
		if (o instanceof UniquePath) {
			UniquePath up2=(UniquePath)o;
			if (up2.edges.size()!=edges.size())
				return false;
			for (int i=0;i<edges.size();i++) {
				if (edges.get(i)!=up2.edges.get(i))
					return false;
			}
			return true; // all same
		} else
			return false;
	}

	// -------------------------------------------
  	// This section is used to set relations of parent/child for UI tree
  	// Three signature methods are defined: getParent,getChildren,hasChildren	

	public void reset() {
		parent=null;
		children.clear();
	}
	

	// Inserts child path to at the begining of the list, used by backward retrieving
	public void insertChild(ITreeNode child) {
		children.add(0,child);
	} 
	
	private MethodPropertySource meps; // for property display
	/** 
	* Class adapter
	* @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	*/
	public Object getAdapter(Class adapter) {
		// TODO Auto-generated method stub
		if (adapter==IPropertySource.class) {
			if (meps==null) {
				// cache
				meps=new MethodPropertySource(this);
			}
			return meps;
		}
		return null;
		}
		
	// return the method name assocaited with node  	
	public String getName() {
		return getNode().getLabel();
	}
	public String getFullQualifiedName() {
		return getNode().getLabel();		
	}
	public String getType() {
		String type="to be decided";
		return type;
	}
	public String getScope() {
		String scope="to be decided";
		return scope;
	}
	public int getNumOfChildren() { // visible
		int counter=0;
		Node node=getNode();
		for (int i=0;i<node.getNumberOfChildren();i++) {
			 if (!node.getChildAt(i).isHidden())
			 	counter++;
		}
		return counter;
	}
	public int getLevelOfCall() {
		return edges.size();	
	}

	/* (non-Javadoc)
	 * @see seat.ui.tree.ITreeNode#inside(java.util.List)
	 */
	public boolean inside(List list) {
		// TODO Auto-generated method stub
		Iterator it=list.iterator();
		while (it.hasNext()) {
			if ( ((Node)it.next()).equals(this.getElement()) )
				return true;
		}
		return false;
	}

}
