/** Generic Algorithm Actions.
 *  Special algorithm will be substitute generic later on.
 */
package seat.ui.views;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.util.Assert;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;

import seat.SeatUtil;
import seat.model.algorithms.Algorithm;
import seat.model.core.dynamic.Trace;
import seat.ui.algorithm.AlgorithmDescription;
import seat.ui.algorithm.AlgorithmList;
import seat.ui.algorithm.Parameter;
import seat.ui.tree.ITraceEditor;

public class AlgorithmAction extends Action {
	protected AlgorithmDescription algorithmDesc; // hold info for alg and para
	
	protected Algorithm algorithm; // create and cahed for performance
//	private AlgorithmList al; 
//	private boolean algorthimApplied=false;
		
	public AlgorithmAction(AlgorithmDescription alg) {
		super("",Action.AS_CHECK_BOX);
		algorithmDesc=alg;
		setId(alg.getID());
		setText(alg.getName());
		setToolTipText(alg.getName());
		setImageDescriptor(SeatUtil.getImageDescriptor(alg.getIcon()));
	}
	
//	// must be called after constrcutor to set list info, else run()
//	// will not work correctly. al is not put as part of constructor because EditorContributor can
//	// not get al info when initiate; it will get and set it in setActiveEdtor event.
//	public void setGroup(AlgorithmList al) {
////		this.al=al;
//	}
	/**
	 * Main method for the action when icon is clicked or command issued.
	 */
	public void run() {
		IEditorPart ep=SeatUtil.getActiveEditor();
		Assert.isNotNull(ep);
		if (ep instanceof ITraceEditor)
			runAlgoirthm((ITraceEditor)ep,false);
		MultiActionContributor.reapply4CurrentEditor(this); // redo other effective algs
	}
	
	/**
	 * Applies or cancels current algorithm; or if update is true, reapply algorithm if it's already in effect
	 * @param update: false, when called by action run method, and the check/uncheck status will alternate.
	 *                true, reapply algorithm to reflect changes if this algorithm is on. 
	 */
	public void runAlgoirthm(ITraceEditor ed,boolean updat) {
		final ITraceEditor editor=ed;
		final boolean update=updat;
		BusyIndicator.showWhile(Display.getCurrent(),new Runnable() {
			public void run() {
				core(editor.getTrace(),update);
				editor.refreshViewerWithReload(); 
				SeatUtil.refreshStatisViewIfOpened();
			}
		});
	} 
	
	/**
	 * apply or reapply algorithm. kermal part with UI processing.
	 * @param trace
	 * @param update
	 */
	public void core(Trace trace,boolean update) {
		// prepare paras
		Parameter[] paras=algorithmDesc.getParameters();
		String index=algorithmDesc.getIndex();
		for (int i=1;i<paras.length;i++) { // o is for internal index
			paras[i].setCurrent(SeatUtil.getStringSetting(index+"@#@"+paras[i].getID()));
		}
		//load algorithm if not yet
		if (algorithm==null) 
			algorithm=algorithmDesc.createAlgorithm();		
		algorithm.init(paras);		
		AlgorithmList al=trace.getState().getAlgorithmList();
		Assert.isNotNull(al);
//		String index=algorithmDesc.getIndex();
		boolean algorthimApplied=al.getAlgorithmStatus(index);
		if (update) {
			if (algorthimApplied) {
				algorithm.undo(trace);
				algorithm.apply(trace);
			}
			return;
		} else if (!algorthimApplied) {
			algorithm.apply(trace);
//			algorthimApplied=true;
			al.setAlgorithmStatus(index,true);			
		} else {
			algorithm.undo(trace);
//			algorthimApplied=false;						
			al.setAlgorithmStatus(index,false);			
		}			
	}
}
