package seat.ui.views;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import seat.SeatPlugin;
import seat.SeatUtil;

public class ControlPanelView extends ViewPart {
	private Button showAll;
	private Button useDecoration;
	
	private IPreferenceStore seatStore=SeatPlugin.getDefault().getPreferenceStore();
	private IPreferenceStore workbenchStore=PlatformUI.getWorkbench().getPreferenceStore();
	/**
	 * Listens to prefrence page property changes
	 */
	private IPropertyChangeListener seatPrefListener =
		new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				// TODO Auto-generated method stub
				String property=event.getProperty();
				if (property.equals(SeatUtil.TRACE_SHOWFILTERED)) {
					showAll.setSelection(((Boolean)event.getNewValue()).booleanValue());
				} else if (property.equals(SeatUtil.TRACE_ENABLED_DECORATORS)) {
//					String decors=(String)event.getNewValue();
					useDecoration.setSelection(DecorationPreference.isTraceDecorationOn());//decors.indexOf("TraceLightDecorator:true")!=-1);					
				} 		
			}		
	};
	
	/** 
	 * Listens to selection changes for chebox boxes in this view
	 * and updates store prefrence settings.
	 */
	private SelectionListener buttonClickListener=new SelectionListener() {
		public void widgetSelected(SelectionEvent e) {
			// TODO Auto-generated method stub
			if (e.getSource() instanceof Button) {
				Button b=(Button)(e.getSource());
				boolean status=b.getSelection();
				if (b==showAll) {	
					seatStore.setValue(SeatUtil.TRACE_SHOWFILTERED,status);			
				} else if (b==useDecoration) {
					DecorationPreference.setTraceDecoration(status);
//					String decors=workbenchStore.getString(SeatUtil.TRACE_ENABLED_DECORATORS);
//					String newDec;
//					if (status)
//						newDec=decors.replaceAll("TraceLightDecorator:false","TraceLightDecorator:true");
//					else
//						newDec=decors.replaceAll("TraceLightDecorator:true","TraceLightDecorator:false");
//					workbenchStore.setValue(SeatUtil.TRACE_ENABLED_DECORATORS,newDec);
				}
				MultiActionContributor.reapplyAll();
			}		 
		}

		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		public void widgetDefaultSelected(SelectionEvent e) {
			// TODO Auto-generated method stub		
		}
	};
	
	public ControlPanelView() {
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		Group up=new Group(parent,SWT.SHADOW_ETCHED_OUT);
		up.setText("Settings");
		GridLayout upl=new GridLayout();
		upl.numColumns=1;
		up.setLayout(upl);
		
		showAll=new Button(up,SWT.CHECK);
		showAll.setText("Show nodes that are filtered by algoirthms");
		showAll.setSelection(seatStore.getBoolean(SeatUtil.TRACE_SHOWFILTERED));
		showAll.addSelectionListener(buttonClickListener);
		
		useDecoration=new Button(up,SWT.CHECK);
		useDecoration.setText("Trace decoration");
		String decors=workbenchStore.getString(SeatUtil.TRACE_ENABLED_DECORATORS);
		useDecoration.setSelection(DecorationPreference.isTraceDecorationOn());//decors.indexOf("TraceLightDecorator:true")!=-1);					
		useDecoration.addSelectionListener(buttonClickListener);
		
		seatStore.addPropertyChangeListener(seatPrefListener);
		workbenchStore.addPropertyChangeListener(seatPrefListener);
		
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
		super.dispose();
		seatStore.removePropertyChangeListener(seatPrefListener);
		workbenchStore.removePropertyChangeListener(seatPrefListener);		
	}

}
