/*
 * Created on May 10, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.ui.views;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.Assert;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.ui.tree.ITraceEditor;

/**
 * Creates an exploration view in the primary layout. The initial location of exploration view 
 * is upper left corner. Multiple explorations are listed in this view. A new exploration can
 * be added to this view by an open action and can be deactived to save memory. All the exploration 
 * can also be deleted.   
 * <p>
 * When current selecteion changed, an event is fired to the TreeView to active current active trace tree.
 * The view also provides session persistence functionality. The previously saved exploration will 
 * automatically be bought up when the platform is launched again.
 * 
 * @author lfu
 * @see org.eclipse.ui.part.ViewPart
 * @see org.kbre.seat.views.TreeView
 */

// data in viewer is Node objects
public class FilterView extends ViewPart implements IPartListener{
	protected TableViewer allViewer,filteredViewer;
	private static int ALL_VIEWER=1;
	private static int FILTERED_VIEWER=2;
	private ArrayList al,fl;

	private Action leftAction,rightAction;
	private Action leftSelection,rightSelection;
	private ITraceEditor editor,previous=null;

	private String xmlTitle;
	
	/**
	 * The content provider class is responsible for
	 * providing objects to the view. The actual objects returned are Explorations 
	 */
	class ViewContentProvider implements IStructuredContentProvider {
		private Object[] sortedNodes;
		private Comparator compare=new Comparator() {
			public int compare(Object o1, Object o2) {
				Node po1 = (Node) o1;
				Node po2 = (Node) o2;
				return po1.getLabel().compareTo(po2.getLabel());
			}
			
		};	
		private int which;		
		public ViewContentProvider(int v) {
			super();
			// TODO Auto-generated constructor stub
			which=v;
		}

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {
			if (which==ALL_VIEWER) 
				sortedNodes=al.toArray();
			else 
				sortedNodes=fl.toArray();
			Arrays.sort(sortedNodes,compare);
			return sortedNodes;
		}
		
		/**
		 * Help method that finds index of given object in the sorted list, if not found, return -1
		 * @author lfu
		 */
		public int indexOf(Object obj) {
			for (int i=0;i<sortedNodes.length;i++)
				if (sortedNodes[i]==obj)
					return i;
				return -1;
		}
		
	}

	public class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		private ArrayList imageCache = new ArrayList(4); // cache for performance

		private int which;		
		public ViewLabelProvider(int v) {
			super();
			// TODO Auto-generated constructor stub
			which=v;
			imageCache.add(SeatUtil.getImageDescriptor("icons/class_obj.gif").createImage());
			imageCache.add(SeatUtil.getImageDescriptor("icons/public_co.gif").createImage());
			imageCache.add(SeatUtil.getImageDescriptor("icons/fields_co.gif").createImage());
			imageCache.add(SeatUtil.getImageDescriptor("icons/inactive.gif").createImage());
		}

		public String getColumnText(Object obj,int index) {
			if (obj instanceof Node)
				return ((Node)obj).getLabel();
			else 
				return obj.toString();
		}

		public Image getColumnImage(Object obj, int index) {
			return getImage(obj);
		}

		/**
		 *  Returns the system image ISharedImages.IMG_OBJ_ELEMENT as the basic image
		 */
		public Image getImage(Object obj) {
			if (which==ALL_VIEWER) {
				Node node=(Node)obj;				
				return (node.getNumberOfChildren()>0)?(Image)imageCache.get(0):(Image)imageCache.get(1);
			} else if (which==FILTERED_VIEWER)
				return (Image)imageCache.get(2);
			else 
				return (Image)imageCache.get(3);
		}	
	}
	
	/**
	 * 
	 */
	public FilterView() {
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		Control viewerControl=allViewer.getControl();
		if (viewerControl!=null && !viewerControl.isDisposed()) 
			viewerControl.setFocus();
	}


	/**
	 * This is a callback that will allow us to create the viewer and initialize it.
	 * 
	 */
	public void createPartControl(Composite parent) {
		Composite composite=parent;
//		Composite composite=new Composite(parent,SWT.NONE);
//		GridLayout layout = new GridLayout();
//		layout.numColumns = 2;
//		composite.setLayout(layout);
//		GridData gridData = new GridData();
////		gridData.horizontalAlignment = GridData.FILL;
//		gridData.grabExcessHorizontalSpace = true;
//		gridData.grabExcessVerticalSpace= true;
//		composite.setLayoutData(gridData);		
		
//		Label l1 = new Label(composite, SWT.LEFT);
//		l1.setText("Visible Node List");
//		Label l2 = new Label(composite, SWT.LEFT);
//		l2.setText("Filtered Node List");
//		GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_CENTER|GridData.VERTICAL_ALIGN_CENTER);
//		gd.widthHint=10;
//		gd.grabExcessVerticalSpace = true;

		allViewer = new TableViewer(composite, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.HIDE_SELECTION);
		allViewer.setContentProvider(new ViewContentProvider(ALL_VIEWER));
		allViewer.setLabelProvider(new ViewLabelProvider(ALL_VIEWER));
//		allViewer.setSorter(new Sorter());
		allViewer.setInput(al);
		
//		addButtons(parent);
		
		filteredViewer = new TableViewer(composite, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.HIDE_SELECTION);
		filteredViewer.setContentProvider(new ViewContentProvider(FILTERED_VIEWER));
		filteredViewer.setLabelProvider(new ViewLabelProvider(FILTERED_VIEWER));
//		filteredViewer.setSorter(new Sorter());
		filteredViewer.setInput(fl);

		// add self as the listener to the editor 
		this.getSite().getPage().addPartListener(this);
		createActions();
		hookContextMenu();
		contributeToActionBars();
		hookDoubleClickAction();	
		xmlTitle=this.getTitle();

		updateActionStatus();
	}

	public void updateTitle(int occ1,int occ2) {
		this.setTitle(xmlTitle+" ("+ occ1+" visible[left window], "+ occ2+" filtered[right window])");		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
		super.dispose();
		this.getSite().getPage().removePartListener(this);
	}

	// Adds context menu
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenuExpl1");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu1(manager);
			}
		});
		Menu menu1 = menuMgr.createContextMenu(allViewer.getControl());
		allViewer.getControl().setMenu(menu1);
		getSite().registerContextMenu(menuMgr, allViewer);

		MenuManager menuMgr2 = new MenuManager("#PopupMenuExpl2");
		menuMgr2.setRemoveAllWhenShown(true);
		menuMgr2.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu2(manager);
			}
		});
		Menu menu2 = menuMgr2.createContextMenu(filteredViewer.getControl());
		filteredViewer.getControl().setMenu(menu2);
		getSite().registerContextMenu(menuMgr2, filteredViewer);
	}
	
	// Adds items to the pulldown menu and toolbar of this view
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalToolBar(bars.getToolBarManager());
	}
	// Hooks to the pulldown, called by hookcontextMenu
	protected void fillContextMenu1(IMenuManager manager) {
		manager.add(leftSelection);
		manager.add(rightAction);
		manager.add(new Separator("Additions")); // a must for context menu
	}

	protected void fillContextMenu2(IMenuManager manager) {
		manager.add(rightSelection);
		manager.add(leftAction);
		manager.add(new Separator("Additions")); // a must for context menu
	}

	/* (non-Javadoc)
	 * @see seat.ui.views.MethodView#fillLocalToolBar(org.eclipse.jface.action.IToolBarManager)
	 */
	protected void fillLocalToolBar(IToolBarManager manager) {
		// TODO Auto-generated method stub
		manager.add(leftAction);
		manager.add(rightAction);
	}	


	private void createActions() {
		// TODO Auto-generated method stub
		leftSelection= new Action() {
			public void run() {
				leftSelectionClicked();
			}
		};
		leftSelection.setText("Select All");
		leftSelection.setAccelerator(SWT.CTRL|'A');
		leftSelection.setToolTipText("Select All");
//		leftSelection.setImageDescriptor(SeatUtil.getImageDescriptor("icons/left.gif"));

		leftAction = new Action() {
			public void run() {
				leftClicked();
			}
		};
		leftAction.setText("Move to Visible List");
		leftAction.setToolTipText("Move to Visible List");
		leftAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/left.gif"));

		rightSelection= new Action() {
			public void run() {
				rightSelectionClicked();
			}
		};
		rightSelection.setText("Select All");
		rightSelection.setAccelerator(SWT.CTRL|'A');
		rightSelection.setToolTipText("Select All");
		
		rightAction = new Action() {
			public void run() {
				rightClicked();
			}
		};
		rightAction.setText("Move to Filtered List");
		rightAction.setToolTipText("Move to Filtered List");
		rightAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/right.gif"));

		
	}
	
	// Adds double click funtion to the viewer
	private void hookDoubleClickAction() {
		allViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (rightAction!=null)
					rightAction.run();
			}
		});
		filteredViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (leftAction!=null)
					leftAction.run();
			}
		});
	}

	public void partActivated(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		if ((part instanceof ITraceEditor)&& (part!=previous)) {
			// remove old list if there
			Assert.isNotNull(allViewer.getControl());
			Assert.isNotNull(filteredViewer.getControl());
			
			Control allControl=allViewer.getControl();			
			Control filControl=filteredViewer.getControl();
			
			if ( (allViewer==null)||(allControl==null)|| (allControl.isDisposed()) ||
				 (filteredViewer==null)||(filteredViewer.getControl()==null)||(filControl.isDisposed()) )
				 return;
			allControl.setRedraw(false);
//			if (al!=null) {
//				Object[] rows=al.toArray();
//				allViewer.remove(rows);
//			}
			//add new
			editor=(ITraceEditor)part;			
			previous=editor;
			al=editor.getTrace().getState().getNodeList();
			allViewer.setInput(al);
			allControl.setRedraw(true);

			filControl.setRedraw(false);
//			if (fl!=null) {
//				Object[] rows=fl.toArray();
//				filteredViewer.remove(rows);
//			}			
			//add new
			fl=editor.getTrace().getState().getFilteredNodeList();
			filteredViewer.setInput(fl);
			filControl.setRedraw(true);

			editor.refreshIconOnly();
			updateTitle(al.size(),fl.size());	
		} 				
	}

	/**
	 * Called by editor init, editor will create this view and set itself as the editor
	 * @param editor
	 */
	public void setEditor(ITraceEditor editor) {
		this.editor=editor;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partClosed(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partClosed(IWorkbenchPart part) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partDeactivated(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partDeactivated(IWorkbenchPart part) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partOpened(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partOpened(IWorkbenchPart part) {
		// TODO Auto-generated method stub

	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partBroughtToTop(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partBroughtToTop(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		partActivated(part);
	}
	public void filterNode(Node[] objs) {
		allViewer.remove(objs);
		filteredViewer.add(objs);
		for (int i=0;i<objs.length;i++) {
			((Node)objs[i]).hide(SeatUtil.RESERVED_ALGIDX_FILTER);
			al.remove(objs[i]);
			fl.add(objs[i]);			
		}
		editor.refreshViewer();		
		updateTitle(al.size(),fl.size());	
		SeatUtil.refreshStatisViewIfOpened();
	}	
	
	private void leftClicked() {
		ISelection selection = filteredViewer.getSelection();
		Object[] objs=((IStructuredSelection)selection).toArray();
		if (objs.length<1)
			return;
		Object last=objs[objs.length-1];
		int[] idx=filteredViewer.getTable().getSelectionIndices();
		int lastIndex=idx[idx.length-1];
		for (int i=0;i<objs.length;i++) {
			((Node)objs[i]).show(SeatUtil.RESERVED_ALGIDX_FILTER);
			fl.remove(objs[i]);
			al.add(objs[i]);			
		}
		filteredViewer.getControl().setRedraw(false);
		filteredViewer.refresh();//setInput(fl);
		int revisedIndex=lastIndex-objs.length+1;
		if (revisedIndex>=fl.size())
			revisedIndex=fl.size()-1;
		if (revisedIndex>=0) {
			filteredViewer.getTable().setSelection(revisedIndex);
		}
		filteredViewer.getControl().setRedraw(true);
		allViewer.getControl().setRedraw(false);
		allViewer.refresh();//setInput(al);
		allViewer.getControl().setRedraw(true);
		editor.refreshViewerWithReload();	
		updateTitle(al.size(),fl.size());	
		updateActionStatus();
		SeatUtil.refreshStatisViewIfOpened();
	}
	
	private void rightClicked() {
		ISelection selection = allViewer.getSelection();
		Object[] objs=((IStructuredSelection)selection).toArray();
		if (objs.length<1)
			return;
		Object last=objs[objs.length-1];
		int[] idx=allViewer.getTable().getSelectionIndices();
		int lastIndex=idx[idx.length-1];
		for (int i=0;i<objs.length;i++) {
			((Node)objs[i]).hide(SeatUtil.RESERVED_ALGIDX_FILTER);
			al.remove(objs[i]);
			fl.add(objs[i]);			
		}
		filteredViewer.getControl().setRedraw(false);
		filteredViewer.refresh();//setInput(fl);
		filteredViewer.getControl().setRedraw(true);
		allViewer.getControl().setRedraw(false);
		allViewer.refresh();//setInput(al);
		int revisedIndex=lastIndex-objs.length+1;
		if (revisedIndex>=al.size())
			revisedIndex=al.size()-1;
		if (revisedIndex>=0) {
			allViewer.getTable().setSelection(revisedIndex);
		}
		allViewer.getControl().setRedraw(true);
//		filteredViewer.setInput(fl);
//		allViewer.setInput(al);
		editor.refreshViewerWithReload();		
		updateTitle(al.size(),fl.size());	
		updateActionStatus();
		SeatUtil.refreshStatisViewIfOpened();

	}		

	private void leftSelectionClicked() {
		allViewer.getTable().selectAll();
	}
			
	private void rightSelectionClicked() {
		filteredViewer.getTable().selectAll();
	}
	
	private void updateActionStatus() {
		if ((al==null)||(fl==null)) // may have not set yet
			return;
		if (al.size()==0) {
			leftSelection.setEnabled(false);
			rightAction.setEnabled(false);
		} else {
			leftSelection.setEnabled(true);
			rightAction.setEnabled(true);			
		}
		if (fl.size()==0) {
			rightSelection.setEnabled(false);
			leftAction.setEnabled(false);
		} else {
			rightSelection.setEnabled(true);
			leftAction.setEnabled(true);			
		}		
	}
}
