/*
 * Created on Jul 15, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.ui.views;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.PropertyPage;

import seat.model.core.structure.MethodElement;
import seat.ui.tree.ITreeNode;

/**
 * Display the property page of a given ui tree node.
 * This page is brought up by right-click a Tree node and select property.
 * In contrast to the automatic property-view.
 *  * @author lfu
 */
public class MethodPropertyPage extends PropertyPage {
	private Text description;
	private ITreeNode node;

	/**
	 * Constructor.
	 */
	public MethodPropertyPage() {
		super();
	}
	
	/**
	 * Creates the property page
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		// TODO Auto-generated method stub
		Composite composite=new Composite(parent,SWT.NONE);
		GridLayout layout=new GridLayout();
		layout.numColumns = 2;
		composite.setLayout(layout);
		GridData data=new GridData(GridData.FILL);
		data.grabExcessHorizontalSpace=true;
		composite.setLayoutData(data);
//		Composite composite=parent;

		IAdaptable owner=getElement();
		if (owner instanceof ITreeNode) {
			node=(ITreeNode)owner;
			addProperty(composite,"Method Name:",node.getName());
			addProperty(composite,"Full Qualified Name:",node.getFullQualifiedName());
			addProperty(composite,"Type:",node.getType());
			addProperty(composite,"Scope:",node.getScope());
			addProperty(composite,"Parent Method:",node.getParent().getFullQualifiedName());
			addProperty(composite,"Number of Direct Children:",node.getNumOfChildren()+"");
			addProperty(composite,"Visibility Level:",node.getLevelOfCall()+"");
			
			addSeparator(composite);
			addCommentProperty(composite,"User Description of the Method:");			
		}
		else 
			addProperty(composite,"Current selection",owner.toString());
		return composite;
	}
	
	// Adds a property to page
	private void addProperty(Composite parent,String propName, String propValue) {
		// create layout first
//		Composite composite = new Composite(parent, SWT.NULL);
//		GridLayout layout = new GridLayout();
//		layout.numColumns = 2;
//		composite.setLayout(layout);
//		GridData data=new GridData(GridData.FILL);
//		data.grabExcessHorizontalSpace=true;
//		composite.setLayoutData(data);
		Composite composite =parent;
		// add property label and value
		Label label=new Label(composite,SWT.NONE);
		label.setText(propName);
		Label attribute=new Label(composite,SWT.NONE);
		attribute.setText(propValue);
	}
	
	// Adds the description property to page
	private void addCommentProperty(Composite parent,String propName) {
		// create layout first
//		Composite composite=new Composite(parent, SWT.NULL);
//		GridLayout layout = new GridLayout();
//		layout.numColumns = 1;
//		parent.setLayout(layout);
//		GridData data=new GridData(GridData.FILL);
//		data.grabExcessHorizontalSpace=true;
//		data.horizontalSpan=2;
//		composite.setLayoutData(data);

		Composite composite =parent;
		// add property label and value
		Label label=new Label(composite,SWT.NONE);
		label.setText(propName);
		GridData data=new GridData();
		data.horizontalAlignment = GridData.FILL;
		data.grabExcessHorizontalSpace=true;
		data.horizontalSpan=2;
		label.setLayoutData(data);
				
		description =new Text(composite,SWT.MULTI|SWT.WRAP|SWT.V_SCROLL|SWT.H_SCROLL);
		GridData gd = new GridData();
		gd.heightHint=180;
		gd.horizontalAlignment = GridData.FILL;
		data.grabExcessHorizontalSpace=true;
		gd.verticalAlignment= GridData.BEGINNING;
		gd.horizontalSpan=2;
		description.setLayoutData(gd);
		MethodElement me=(MethodElement)node.getElement().getBehaviouralElement();
		String desc=me.getDescription();	
		if (desc==null)
			desc="";
		description.setText(desc);

	}

	// Adds a separator
	private void addSeparator(Composite parent) {
		Label separator = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan=2;
		separator.setLayoutData(gridData);
	}
	
	// Creates a composite	
	private Composite createDefaultComposite(Composite parent) {
//		Composite composite = new Composite(parent, SWT.NULL);
		Composite composite =parent;
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout(layout);

//		GridData data = new GridData();
//		data.verticalAlignment = GridData.FILL;
//		data.horizontalAlignment = GridData.FILL;
//		composite.setLayoutData(data);

		return composite;
	}	
	
	
	/**
	 * Called when the Apply button is pressed.
	 * @see org.eclipse.jface.preference.PreferencePage#performApply()
	 */
	protected void performApply() {
		// TODO Auto-generated method stub
		node.setDescription(description.getText());
//		super.performApply();
	}

	/**
	 * Called when the Restore Defaults button is pressed
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	protected void performDefaults() {
		// TODO Auto-generated method stub
		description.setText("");
//		super.performDefaults();
	}

	/**
	 * Called when the OK button is pressed and changes are saved.
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 */
	public boolean performOk() {
		// TODO Auto-generated method stub
		performApply();
		return super.performOk();
	}

}
