/*
 * Created on Jul 11, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package seat.ui.views;

import java.util.ArrayList;
import java.util.Arrays;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import seat.SeatUtil;
import seat.io.TraceState;
import seat.model.core.dynamic.Node;
import seat.ui.algorithm.NameParsing;
import seat.ui.tree.ITreeNode;

/**
 * This class is used to add system property capability.
 * To make this attribute works, add the following line to the ViewPart
 * createPartControl() that provides the attributes:
 * 		getSite().setSelectionProvider(viewer);
 * @author lfu
 * @see UniquePath
 */
public class MethodPropertySource implements IPropertySource {

	private IPropertyDescriptor[] propertyDescriptors;
	private ITreeNode path;
	
	private static String[] pls;
	private static String[] cls;
	/**
	 * Constructor. Use UniquePath as the source input of property page
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public MethodPropertySource(ITreeNode source){
		path=source;
		if (path.getTrace()!=null) {
			ArrayList plu=path.getTrace().getState().getPackageList();
			pls=(String[])plu.toArray(new String[plu.size()]);
			Arrays.sort(pls);
			ArrayList clu=path.getTrace().getState().getClassList();
			cls=(String[])clu.toArray(new String[clu.size()]);
			Arrays.sort(cls);
		}
	}

	/**
	 * Returns editable value for a property
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		// TODO Auto-generated method stub
		return this;
	}

	/**
	 * Creates descriptors for all the property
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		// TODO Auto-generated method stub
		if (propertyDescriptors==null) {			
			
			PropertyDescriptor traceNodes=new PropertyDescriptor(SeatUtil.PROPERTY_TRACE_NODES,"Total Nodes");
			PropertyDescriptor traceGraph=new PropertyDescriptor(SeatUtil.PROPERTY_TRACE_GRAPH,"Nodes in Graph");
			PropertyDescriptor traceCompressed=new PropertyDescriptor(SeatUtil.PROPERTY_TRACE_COMPRESSED,"UI Tree Nodes Compressed");
			PropertyDescriptor traceLoad=new PropertyDescriptor(SeatUtil.PROPERTY_TRACE_LOADTIME,"Loading Time");
			PropertyDescriptor tracePatterns=new PropertyDescriptor(SeatUtil.PROPERTY_TRACE_PATTERNS,"Pattern Detected");
			PropertyDescriptor traceSource=new PropertyDescriptor(SeatUtil.PROPERTY_TRACE_SOURCE,"Trace Source");
			PropertyDescriptor traceType=new PropertyDescriptor(SeatUtil.PROPERTY_TRACE_TYPE,"Tace Type");
			PropertyDescriptor tracePackage=new PropertyDescriptor(SeatUtil.PROPERTY_TRACE_PACKAGE,"Total Packages");
			PropertyDescriptor traceClass=new PropertyDescriptor(SeatUtil.PROPERTY_TRACE_CLASS,"Total classes");
			traceNodes.setCategory("Trace Statistics");
			traceGraph.setCategory("Trace Statistics");
			traceCompressed.setCategory("Trace Statistics");
			traceLoad.setCategory("Trace Statistics");
			tracePatterns.setCategory("Trace Statistics");
			traceSource.setCategory("Trace Statistics");
			traceType.setCategory("Trace Statistics");
			tracePackage.setCategory("Trace Statistics");
			traceClass.setCategory("Trace Statistics");
			
			PropertyDescriptor[] pckList=null;
			PropertyDescriptor[] clList=null;;
			if (path.getTrace()!=null) {
				pckList=new PropertyDescriptor[pls.length];
				int len=(pls.length+"").length();
				for (int i=0;i<pls.length;i++) {
					int il=(""+(i+1)).length();
					String id="";
					for (int k=0;k<len-il;k++)
						id+="0";
					id+=(i+1);
					pckList[i]=new PropertyDescriptor(pls[i],"");// used to be index: id);
					pckList[i].setCategory("Package List");
				}
				clList=new PropertyDescriptor[cls.length];
				len=(cls.length+"").length();
				for (int i=0;i<cls.length;i++) {
					int il=(""+(i+1)).length();
					String id="";
					for (int k=0;k<len-il;k++)
						id+="0";
					id+=(i+1);
					clList[i]=new PropertyDescriptor(cls[i],"");//// used to be index: id);id);
					clList[i].setCategory("Class List");
				}
			}
			
			PropertyDescriptor name=new PropertyDescriptor(SeatUtil.PROPERTY_NAME,"Method Name");
			PropertyDescriptor fullname=new PropertyDescriptor(SeatUtil.PROPERTY_FULLNAME,"Full Qualified Name");
//			PropertyDescriptor type=new PropertyDescriptor(SeatUtil.PROPERTY_TYPE,"Type");		
			PropertyDescriptor nparents=new PropertyDescriptor(SeatUtil.PROPERTY_NPARENTS,"Called By");		
			PropertyDescriptor nchildren=new PropertyDescriptor(SeatUtil.PROPERTY_NCHILDREN,"Calls");		
//			PropertyDescriptor scope=new PropertyDescriptor(SeatUtil.PROPERTY_SCOPE,"Scope");		
			PropertyDescriptor parent=new PropertyDescriptor(SeatUtil.PROPERTY_PARENT,"Parent Method");		
//			PropertyDescriptor children=new PropertyDescriptor(SeatUtil.PROPERTY_CHILDREN,"Number of Direct Children");		
			PropertyDescriptor level=new PropertyDescriptor(SeatUtil.PROPERTY_LEVEL,"Level");		
			PropertyDescriptor description=new TextPropertyDescriptor(SeatUtil.PROPERTY_DESCRIPTION,"Description");		
			name.setCategory("Current Node");
			fullname.setCategory("Current Node");
//			type.setCategory("Current Node");
			nparents.setCategory("Current Node");
			nchildren.setCategory("Current Node");
//			scope.setCategory("Current Node");
			parent.setCategory("Current Node");
//			children.setCategory("Current Node");
			level.setCategory("Current Node");
			description.setCategory("Current Node");
			
//			propertyDescriptors=new IPropertyDescriptor[] {
//				traceNodes,traceGraph,traceCompressed,traceLoad,tracePatterns,traceSource,traceType,tracePackage,traceClass,name,fullname,type,scope,parent,children,level,description };
			ArrayList pds=new ArrayList();
			if (SeatUtil.getBooleanSetting("temp.prop")) {
				pds.add(name);
				pds.add(fullname);
//				pds.add(type);
				pds.add(nparents);
				pds.add(nchildren);
//				pds.add(scope);
				pds.add(parent);
//				pds.add(children);
				pds.add(level);
				pds.add(description);				
			} else { 
			pds.add(traceNodes);
			pds.add(traceGraph);
			pds.add(traceCompressed);
			pds.add(traceLoad);
			pds.add(tracePatterns);
			pds.add(traceSource);
			pds.add(traceType);
			pds.add(tracePackage);			
			pds.add(traceClass);
			for (int i=0;i<pckList.length;i++)
				pds.add(pckList[i]);
			for (int i=0;i<clList.length;i++)
				pds.add(clList[i]);
			pds.add(name);
			pds.add(fullname);
//			pds.add(type);
			pds.add(nparents);
			pds.add(nchildren);
//			pds.add(scope);
			pds.add(parent);
//			pds.add(children);
			pds.add(level);
			pds.add(description);
		}
			propertyDescriptors=(IPropertyDescriptor[])pds.toArray(new IPropertyDescriptor[pds.size()]);
		}
		return propertyDescriptors;
	}

	/**
	 * Returns a value by proprty id. The value is to be displayed.
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {
		if (path.getTrace()!=null) {
			TraceState state=path.getTrace().getState();
			if (id.equals(SeatUtil.PROPERTY_TRACE_NODES))
				return state.getTreeNodes()+"";
			if (id.equals(SeatUtil.PROPERTY_TRACE_GRAPH))
				return state.getGraphSize()+"";
			if (id.equals(SeatUtil.PROPERTY_TRACE_COMPRESSED))
				return state.getCompressedNodes()+"";
			if (id.equals(SeatUtil.PROPERTY_TRACE_LOADTIME))
				return state.getLoadTime()+"sec";
			if (id.equals(SeatUtil.PROPERTY_TRACE_PATTERNS))
				return state.getDetectedPatterns()+"";
			if (id.equals(SeatUtil.PROPERTY_TRACE_SOURCE))
				return state.getSource().getName();
			if (id.equals(SeatUtil.PROPERTY_TRACE_TYPE)) {
				int type=state.getType();
				switch (type) {
					case SeatUtil.TRACE_C:
						return "C";
					case SeatUtil.TRACE_JAVA:
						return "JAVA";
					case SeatUtil.TRACE_OO:
						return "Object Oriented";
					case SeatUtil.TRACE_ROUTINE:
						return "Routine";
					default:
						return "Undetermined";
				}
			}
			if (id.equals(SeatUtil.PROPERTY_TRACE_PACKAGE))
				return state.getPackageList().size()+"";			
			if (id.equals(SeatUtil.PROPERTY_TRACE_CLASS))
				return state.getClassList().size()+"";			
			for (int i=0;i<pls.length;i++)
				if (id.equals(pls[i]))
					return pls[i];
			for (int i=0;i<cls.length;i++)
				if (id.equals(cls[i]))
					return cls[i];
		}
		
		if (id.equals(SeatUtil.PROPERTY_NAME))
			return path.getName();
		if (id.equals(SeatUtil.PROPERTY_FULLNAME))
			return path.getFullQualifiedName();
		if (id.equals(SeatUtil.PROPERTY_TYPE))
			return path.getType();
		if (id.equals(SeatUtil.PROPERTY_NPARENTS)) {
			String par=getDistinctParents(path.getElement())+" distinct method(s): ";
			Node node=path.getElement();
			for (int i=0;i<node.getNumberOfParents();i++) {
				par+=NameParsing.getShortClassName(node.getParentAt(i))+"."+ NameParsing.getShortMethodName(node.getParentAt(i));
				par+="/";
			}
			return par;
//			return getDistinctParents(path.getElement())+" distinct method(s): ";			
		}
		if (id.equals(SeatUtil.PROPERTY_NCHILDREN)) {
			String par=getDistinctChildren(path.getElement())+" distinct method(s): ";
			Node node=path.getElement();
			for (int i=0;i<node.getNumberOfChildren();i++) {
				par+=NameParsing.getShortClassName(node.getChildAt(i))+"."+ NameParsing.getShortMethodName(node.getChildAt(i));
				par+="/";
			}
			return par;
		}
//			return getDistinctChildren(path.getElement())+" distinct method(s)";
		if (id.equals(SeatUtil.PROPERTY_SCOPE))
			return path.getScope();
		if (id.equals(SeatUtil.PROPERTY_PARENT)) 
			if (path.getParent()!=null)
				return path.getParent().getFullQualifiedName();
			else return "";
		if (id.equals(SeatUtil.PROPERTY_CHILDREN))
			return path.getNumOfChildren()+"";
		if (id.equals(SeatUtil.PROPERTY_LEVEL))
			return path.getLevelOfCall()+"";
		if (id.equals(SeatUtil.PROPERTY_DESCRIPTION))
			return path.getDescription();
				
		return null;

	}

	/**
	 * Test if the property is set. Always return false.
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		// TODO Auto-generated method stub
		return false;
	}

	/**
	 * Reset the property to its default value.Empty method.
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub

	}

	/**
	 * Set a property value. Empty method.
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
		// TODO Auto-generated method stub

	}
	
	private int getDistinctChildren(Node node) {
		int n=node.getNumberOfChildren();
		if (n==0) return 0;
		else if (n==1) return 1;
		else {
			ArrayList al=new ArrayList();
			for (int i=0;i<n;i++)
				if (!al.contains(node.getChildAt(i)))
					al.add(node.getChildAt(i));
			return al.size();			
		}
	}
	
	private int getDistinctParents(Node node) {
		int n=node.getNumberOfParents();
		if (n==0) return 0;
		else if (n==1) return 1;
		else {
			ArrayList al=new ArrayList();
			for (int i=0;i<n;i++)
				if (!al.contains(node.getParentAt(i)))
					al.add(node.getParentAt(i));
			return al.size();			
		}
	}	

}
