package seat.ui.views;


import java.util.ArrayList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.model.core.structure.MethodElement;
import seat.ui.algorithm.NameParsing;
import seat.ui.tree.ITraceEditor;

/**
 * Base class for views that needs a tableviewer. The data in the table is Node class.
 * This view class implements ViewContentProvider, ViewLableProvider, ViewSelectionChangeListner
 * and NodeSorter as its helper classes.
 * @author lfu
 * @see org.kbre.seat.views.TreeView
 */
// this clas can be refactored to conenct to active editor at runtime
public abstract class MethodView extends ViewPart implements IPartListener {
	protected TableViewer viewer;
	protected ArrayList list;  // contains either Utility(UniquePath) or PatternOccurrence	
	protected ITraceEditor editor,previous=null;
	
	private TraceSearch search;
	private String searchString;
	private String xmlTitle;

	// all the actions will be created in makeActions();
	private Action nextAction;
	private Action prevAction;
	private Action promoteAction;
	private Action sortAction;
	private Action deleteAction;
	private Action deleteAllAction;
	private Action doubleClickAction;
	private NodeSorter occurSorter;  // specifically extra for sort action
	
	class ViewContentProvider implements IStructuredContentProvider {
		// update local tool bar status	
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}

		// Input from ArrayList, returned as an array.
		public Object[] getElements(Object parent) {
			return list.toArray();
		}
	}
	
	/**
	 * Provides labels and images for EUtilityOccurrence objects in view.
	 * @see org.eclipse.jface.viewers.LabelProvider
	 */
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		private Image img=SeatUtil.getImageDescriptor("icons/utility.gif").createImage();
		/** 
		 * Returns the source name of Exploration as labels
		 * See Exploration 
		 */
		public String getColumnText(Object obj, int index) {
			if (obj instanceof Node) {
					Node node=(Node)obj;
					switch (index) {
						case 0:
							return " ";
						case 1:
							return node.getLabel();
						case 2:
							return node.getNumberOfParents()+"";
						case 3:
							MethodElement me=(MethodElement)node.getBehaviouralElement();
							return me.getDescription();	
//							if (editor instanceof MultiTraceEditor) {
//								return ((MultiTraceEditor)editor).getComment(node);
//							} else if (editor instanceof TraceEditor) {
//								return ((TraceEditor)editor).getComment(node);
//							} else 
//								return "";
						default:
							return "";
					}
			} else
					return obj.toString();				
				
		}

		/** 
		 * Returns a image for a given column for the exploration view. The primary viewer in
		 * exploration view is a table viewer.   
		 */
		public Image getColumnImage(Object obj, int index) {
			return (index==0)?img:null;
		}
	}

	class NodeSorter extends ViewerSorter {
		/*
		 * Constructor argument values that indicate to sort items by 
		 * description, owner or percent complete.
		 */
		private final static int NAME			= 1;
		private final static int OCCURRENCE		= 2;
		private final static int DESCRIPTION 	= 3;
		private final static int NAME2			= 11;
		private final static int OCCURRENCE2	= 12;
		private final static int DESCRIPTION2 	= 13;

		// Criteria that used to identify a comaprison
		private int criteria;

		public NodeSorter(int criteria) {
			super();
			this.criteria = criteria;
		}

		/* (non-Javadoc)
		 * Method declared on ViewerSorter.
		 */
		public int compare(Viewer viewer, Object o1, Object o2) {

			Node po1 = (Node) o1;
			Node po2 = (Node) o2;

			switch (criteria) {
				case NAME:
					return compareName(po1, po2);
				case OCCURRENCE:
					return compareOccurrence(po1, po2);
				case DESCRIPTION :
					return compareDescription(po1, po2);
				case NAME2:
					return compareName(po2, po1);
				case OCCURRENCE2:
					return compareOccurrence(po2, po1);
				case DESCRIPTION2 :
					return compareDescription(po2, po1);
				default:
					return 0;
			}
		}

		private int compareName(Node po1, Node po2) {
			return collator.compare(po1.getLabel(), po2.getLabel());
		}
		private int compareOccurrence(Node po1, Node po2) {
			return po1.getNumberOfParents() - po2.getNumberOfParents();
		}
		private int compareDescription(Node po1, Node po2) {
			return 0;//collator.compare(po1.getDescription(), po2.getDescription());
		}
	}
	
	private ISelectionChangedListener viewSelectionChangedListener 
	    =new ISelectionChangedListener() {
		public void selectionChanged(SelectionChangedEvent event) {
			// TODO Auto-generated method stub
			IStructuredSelection selection=(IStructuredSelection)event.getSelection();
			Object first=selection.getFirstElement();
			if ( (first instanceof Node) &&(editor!=null)){
				updateColoredNode((Node)first);
				doubleClickAction.run(); // perform a first search
				updateActionStatus();
			}
		}
	};
	/**
	 * 
	 */
	public MethodView() {
		super();
		// TODO Auto-generated constructor stub
		// TODO Auto-generated constructor stub
		IEditorPart edit=SeatUtil.getActiveEditor();
		if ( (edit!=null)&& (edit instanceof ITraceEditor) ) {
			editor=(ITraceEditor)edit;
			search=new TraceSearch(editor);
			installView();
		}				
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize it.
	 * 
	 */
	public void createPartControl(Composite parent) {
		int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | 
					SWT.FULL_SELECTION | SWT.HIDE_SELECTION;		
		viewer = new TableViewer(parent, style); //SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.HIDE_SELECTION);
		Table table=(Table)viewer.getControl();
		table.setLinesVisible (true);
		table.setHeaderVisible (true);
		String[] titles = {"!", "Name", "Occurrence", "Description" };
		
		TableColumn column = new TableColumn (table, SWT.CENTER,0);
		column.setText ("!");
		column.setWidth (20);
		column = new TableColumn (table, SWT.LEFT,1);
		column.setText ("Name");
		column.setWidth (250);
		column = new TableColumn (table, SWT.LEFT,2);
		column.setText ("Occurrence");
		column.setWidth (100);
		column = new TableColumn (table, SWT.CENTER,3);
		column.setText ( "Description");
		column.setWidth (300);

		viewer.setUseHashlookup(true);		
		viewer.setColumnProperties(titles);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.addSelectionChangedListener(viewSelectionChangedListener);
		setViewerSorter();
		viewer.setInput(list);		
		// add self as the listener to the editor 
		this.getSite().getPage().addPartListener(this);				

		// actions&menus
		makeActions();
		hookContextMenu();
		contributeToActionBars();
		hookDoubleClickAction();
		xmlTitle=this.getTitle();
		if (list!=null && list.size()>0)
			updateTitle(list.size());
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
		super.dispose();
		this.getSite().getPage().removePartListener(this);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		Control viewerControl=viewer.getControl();
		if (viewerControl!=null && !viewerControl.isDisposed()) 
			viewerControl.setFocus();
	}
	

	// part listener interface, partActivated() need to be implemented in subclases.
	public void partActivated(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		if ((part instanceof ITraceEditor)&& (part!=previous)) {
			editor=(ITraceEditor)part;
			previous=editor;
			search=new TraceSearch((ITraceEditor)editor);

			// remove old list if there
			Control viewerControl=viewer.getControl();
			if (viewerControl!=null && !viewerControl.isDisposed()) {
				viewerControl.setRedraw(false);
				if (list!=null) {
					Object[] rows=list.toArray();
					viewer.remove(rows);
				}
			}
			
			//add new
			installView();

			if (viewerControl!=null && !viewerControl.isDisposed()) {
					viewer.setInput(list);
					viewerControl.setRedraw(true);
				}

			editor.refreshIconOnly();
			updateTitle(list.size());						
		}		
	}
	public void partClosed(IWorkbenchPart part) {}
	public void partDeactivated(IWorkbenchPart part) {}
	public void partOpened(IWorkbenchPart part) {}
	public void partBroughtToTop(IWorkbenchPart part) {	partActivated(part); }

	
	/**
	 * Called by outsied interface to add a row to this viewer
	 * @param row- Node type
	 */
	public void addRow(Node row) {
		if (row!=null) 
			viewer.add(row);
	}


	public void updateTitle(int occ) {
		this.setTitle(xmlTitle+" ("+ occ+" occurrences identified)");		
		updateActionStatus();
	}

	// Creates actions defined in the class
	protected void makeActions() {
		// TODO Auto-generated method stub
		doubleClickAction = new Action() {  // action inherit from parent class
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection)selection).getFirstElement();
				String name=((Node)obj).getLabel();
				if (search==null) return;
				IEditorPart ed=SeatUtil.getActiveEditor();
				if (ed instanceof ITraceEditor) {
					search.setEditor((ITraceEditor)ed);
					search.setResponsive(true);
					search.setExact(true);
					search.searchFirst(name);
					searchString=name;
				}
				doubleClicked();
			}
		};			
		// promote to global
		nextAction = new Action() {
			public void run() {
				if (searchString==null)
					SeatUtil.showMessage("Select a row before continue.");
				else { // go to next in the current
					if (search==null) return;
					IEditorPart ed=SeatUtil.getActiveEditor();
					if (ed instanceof ITraceEditor) {
						search.setEditor((ITraceEditor)ed);
						search.searchNext();
					}					
				}
				nextActionClicked();				
			}
		};
		nextAction.setText("Go to Next Occurrence");
		nextAction.setToolTipText("Go to Next Occurrence");
		nextAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/next.gif"));
		
		prevAction = new Action() {
			public void run() {
				if (searchString==null)
					SeatUtil.showMessage("Select a row before continue.");
				else { // go to prev in the current
					if (search==null) return;
					IEditorPart ed=SeatUtil.getActiveEditor();
					if (ed instanceof ITraceEditor) {
						search.setEditor((ITraceEditor)ed);
						search.searchPrevious();
					}															
				}		
				prevActionClicked();		
			}
		};
		prevAction.setText("Go to Previous Occurrence");
		prevAction.setToolTipText("Go to Previous Occurrence");
		prevAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/prev.gif"));

		promoteAction = new Action() {
			public void run() {
				IStructuredSelection sel=(IStructuredSelection)viewer.getSelection();
				Object[] selections=sel.toArray();
				for (int i=0;i<selections.length;i++) 
					promoteActionCliked((Node)selections[i]);								
			}
		};
		promoteAction.setText("Promote to Global");
		promoteAction.setToolTipText("Promote to Global");
		promoteAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/promote.gif"));
			
		// Sort
		sortAction = new Action() {
			public void run() {
				viewer.setSorter(occurSorter);
				viewer.refresh();
				sortActionClicked();
			}
		};
		sortAction.setText("Sort by Occurrences");
		sortAction.setToolTipText("Sort by Occurrences");
		sortAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/sort.gif"));

		// delete
		deleteAction = new Action() {
			public void run() {
				deleteActionClicked();
				if (list==null) return;
				IStructuredSelection sel=(IStructuredSelection)viewer.getSelection();
				Object[] selections=sel.toArray();
				viewer.remove(selections);
				list.removeAll(sel.toList());
				if (SeatUtil.getActiveEditor()==editor)
					editor.refreshIconOnly();
				updateColoredNode(null);
				updateTitle(list.size());
			}
		};
		deleteAction.setText("Delete");
		deleteAction.setToolTipText("Delete This");
		deleteAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/remove_exc.gif"));

		// delete all
		deleteAllAction = new Action() {
			public void run() {
				deleteAllActionClicked();
				if (list==null) return;
				Object[] selections=list.toArray();
				viewer.remove(selections);
				list.clear();
				if (SeatUtil.getActiveEditor()==editor)
					editor.refreshIconOnly();
				updateColoredNode(null);
				updateTitle(0);
			}
		};
		deleteAllAction.setText("Delete All");
		deleteAllAction.setToolTipText("Delete All");
		deleteAllAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/removea_exc.gif"));
						
	
	}

	// Adds context menu
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenuExpl");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				MethodView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	// Adds double click funtion to the viewer
	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (doubleClickAction!=null)
					doubleClickAction.run();
			}
		});
	}

	// Adds items to the pulldown menu and toolbar of this view
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	// Hooks to the pulldown, called by hookcontextMenu
	protected void fillContextMenu(IMenuManager manager) {
		manager.add(promoteAction);
		manager.add(sortAction);
		manager.add(deleteAction);
		manager.add(deleteAllAction);
		manager.add(new Separator("Additions")); // a must for context menu		
	}
	
	// Adds actions to the local pulldown menu
	protected void fillLocalPullDown(IMenuManager manager) {
		// TODO Auto-generated method stub
		// no extra for now
	} 

	// Adds actions to local tool bar 
	protected void fillLocalToolBar(IToolBarManager manager){
		manager.add(nextAction);
		manager.add(prevAction);
		manager.add(promoteAction);
		manager.add(deleteAction);
		manager.add(deleteAllAction);
	}

	// update action enablement for the local toolbar of the view
	private void updateActionStatus(){
		if ((list==null)||(list.size()==0)) { // may have not set yet
			nextAction.setEnabled(false);
			prevAction.setEnabled(false);
			promoteAction.setEnabled(false);
			sortAction.setEnabled(false);
			deleteAction.setEnabled(false);
			deleteAllAction.setEnabled(false);				
			return;
		}

		IStructuredSelection sel=(IStructuredSelection)viewer.getSelection();
		if (sel.getFirstElement()!=null) {
			nextAction.setEnabled(true);
			prevAction.setEnabled(true);
			promoteAction.setEnabled(true);
			deleteAction.setEnabled(true);			
		}
		sortAction.setEnabled(true);
		deleteAllAction.setEnabled(true);				
	}
	
	// set sorted for the table
	private void setViewerSorter() {
		final NodeSorter nameSorter,descSorter;
		final NodeSorter nameSorter2, occurSorter2,descSorter2;			
		nameSorter=new NodeSorter(NodeSorter.NAME);
		occurSorter=new NodeSorter(NodeSorter.OCCURRENCE);
		descSorter=new NodeSorter(NodeSorter.DESCRIPTION);
		nameSorter2=new NodeSorter(NodeSorter.NAME2);
		occurSorter2=new NodeSorter(NodeSorter.OCCURRENCE2);
		descSorter2=new NodeSorter(NodeSorter.DESCRIPTION2);
		
		Table table=(Table)viewer.getControl();
		TableColumn dot=table.getColumn(0);
		dot.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				viewer.getControl().setRedraw(false);
				viewer.setSorter(null);
				viewer.getControl().setRedraw(true);
				viewer.refresh();
			}
		});		
		
		TableColumn name=table.getColumn(1);
		name.addSelectionListener(new SelectionAdapter() {
			private boolean clicked=false;
			public void widgetSelected(SelectionEvent e) {
				viewer.getControl().setRedraw(false);
				if (clicked) {
					viewer.setSorter(nameSorter);
					clicked=false;
				} else {
					viewer.setSorter(nameSorter2);
					clicked=true;					
				}
				viewer.getControl().setRedraw(true);
				viewer.refresh();
			}
		});
		TableColumn occurrence=table.getColumn(2);
		occurrence.addSelectionListener(new SelectionAdapter() {
			private boolean clicked=false;
			public void widgetSelected(SelectionEvent e) {
				viewer.getControl().setRedraw(false);
				if (clicked) {
					viewer.setSorter(occurSorter);
					clicked=false;
				} else {
					viewer.setSorter(occurSorter2);
					clicked=true;					
				}
				viewer.getControl().setRedraw(true);
				viewer.refresh();
			}
		});
		TableColumn description=table.getColumn(3);
		description.addSelectionListener(new SelectionAdapter() {
			private boolean clicked=false;
			public void widgetSelected(SelectionEvent e) {
				viewer.getControl().setRedraw(false);
				if (clicked) {
					viewer.setSorter(descSorter);
					clicked=false;
				} else {
					viewer.setSorter(descSorter2);
					clicked=true;					
				}
				viewer.getControl().setRedraw(true);
				viewer.refresh();
			}
		});
		
	}

	// Extra hooks for each action
	public void doubleClicked(){}
	public void nextActionClicked(){}
	public void prevActionClicked(){}
	public void sortActionClicked(){}
	public void deleteActionClicked(){}	
	public void deleteAllActionClicked(){}
	/**
	 * Promote the given node to be global applicable to all traces. Possible use is to 
	 * save the node information.
	 * @param node- node to globalized.
	 */
	public abstract void promoteActionCliked(Node node);

	/**
	 * Updates colors of tree nodes whoes element matches the given parameter in the editor.
	 * Used delete/deleteall action and selection change event
	 * @param node- element node to be colored
	 */
	public abstract void updateColoredNode(Node node);
	
	/**
	 * Called by Constructor and partActivate() to connect this view to the editor 
	 * and set up input-list data from trace contained in the editor.
	 * editor must be non-empty:use Assert to check.
	 */
	public abstract void installView();
	
}
