/*
 * Created on May 10, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.ui.views;


import java.util.ArrayList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.part.ViewPart;

import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.model.core.structure.MethodElement;
import seat.ui.tree.ITraceEditor;
import seat.ui.tree.ITreeNode;
import seat.ui.tree.TraceViewer;

/**
 * Creates an exploration view in the primary layout. The initial location of exploration view 
 * is upper left corner. Multiple explorations are listed in this view. A new exploration can
 * be added to this view by an open action and can be deactived to save memory. All the exploration 
 * can also be deleted.   
 * <p>
 * When current selecteion changed, an event is fired to the TreeView to active current active trace tree.
 * The view also provides session persistence functionality. The previously saved exploration will 
 * automatically be bought up when the platform is launched again.
 * 
 * @author lfu
 * @see org.eclipse.ui.part.ViewPart
 * @see org.kbre.seat.views.TreeView
 */


// Object in the view will be :StructuredSelection
public class SearchView extends ViewPart {
	private TableViewer viewer;
	private TraceSearch search;
	private ArrayList list;  // contains either Utility(UniquePath) or PatternOccurrence
	
	private Action firstAction;
	private Action lastAction;
	private Action nextAction;
	private Action prevAction;
	private Action doubleClickAction;

	private String xmlTitle;
	/**
	 * The content provider class is responsible for
	 * providing objects to the view. The actual objects returned are Explorations 
	 */
	class ViewContentProvider implements IStructuredContentProvider {
		
		/**
		 * Empty implementation for interface IStructuredContentProvider.
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider
		 */
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}

		/**
		 * Empty implementation for interface IStructuredContentProvider.
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider
		 */
		public void dispose() {
		}

		/**
		 * Gets the input objects for the pattern view. All PatternOccurrence 
		 * are returned by the methods as an array.
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider
		 */
		public Object[] getElements(Object parent) {
			return list.toArray();
			//return new String[] { "One", "Two", "Three" };
		}
	}
	
	
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		private Image img=SeatUtil.getImageDescriptor("icons/method.gif").createImage();

		/** 
		 * Returns the source name of Exploration as labels
		 * See Exploration 
		 */
		public String getColumnText(Object obj, int index) {
			if (obj instanceof StructuredSelection) {
					ITreeNode tn=(ITreeNode)((StructuredSelection)obj).getFirstElement();
					switch (index) {
						case 0:
							return " ";
						case 1:
							return tn.getFullQualifiedName();
						case 2:
							return "1";
						case 3:
							MethodElement me=(MethodElement)(tn.getElement().getBehaviouralElement());
							return me.getDescription();	
						default:
							return "";
					}
			} else
					return obj.toString();				
				
		}

		/** 
		 * Returns a image for a given column for the exploration view. The primary viewer in
		 * exploration view is a table viewer.   
		 */
		public Image getColumnImage(Object obj, int index) {
			return (index==0)?img:null;
		}
	}

	class UtilitySorter extends ViewerSorter {

		/**
		 * Constructor argument values that indicate to sort items by 
		 * description, owner or percent complete.
		 */
		public final static int NAME			= 1;
		public final static int OCCURRENCE		= 2;
		public final static int DESCRIPTION 	= 3;
		public final static int NAME2			= 11;
		public final static int OCCURRENCE2		= 12;
		public final static int DESCRIPTION2 	= 13;

		// Criteria that the instance uses 
		private int criteria;

		public UtilitySorter(int criteria) {
			super();
			this.criteria = criteria;
		}

		/* (non-Javadoc)
		 * Method declared on ViewerSorter.
		 */
		public int compare(Viewer viewer, Object o1, Object o2) {

			ITreeNode po1 = (ITreeNode)((StructuredSelection)o1).getFirstElement();
			ITreeNode po2 = (ITreeNode)((StructuredSelection)o2).getFirstElement();

			switch (criteria) {
				case NAME:
					return compareName(po1, po2);
				case OCCURRENCE:
					return compareOccurrence(po1, po2);
				case DESCRIPTION :
					return compareDescription(po1, po2);
				case NAME2:
					return compareName(po2, po1);
				case OCCURRENCE2:
					return compareOccurrence(po2, po1);
				case DESCRIPTION2 :
					return compareDescription(po2, po1);
				default:
					return 0;
			}
		}

		private int compareName(ITreeNode po1, ITreeNode po2) {
			return collator.compare(po1.getElement().getLabel(), po2.getElement().getLabel());
		}
		private int compareOccurrence(ITreeNode po1, ITreeNode po2) {
			return po1.getElement().getNumberOfParents() - po2.getElement().getNumberOfParents();
		}
		private int compareDescription(ITreeNode po1, ITreeNode po2) {
			return collator.compare(po1.getDescription(), po2.getDescription());
		}
	}
	
	private ISelectionChangedListener viewSelectionChangedListener 
		=new ISelectionChangedListener() {
		public void selectionChanged(SelectionChangedEvent event) {
			// TODO Auto-generated method stub
			doubleClickAction.run(); // perform a first search
		}
	};
		
	/**
	 * 
	 */
	public SearchView() {
		super();
		// TODO Auto-generated constructor stub
	}
	// must be set after view is created
	public void setSearch(TraceSearch ts) {
		search=ts;		
		list=ts.getSearch();
		viewer.setInput(list);
		if (list ==null || list.size()==0)
			firstAction.setEnabled(false);
		else 
			firstAction.setEnabled(true);
		lastAction.setEnabled(false);
		nextAction.setEnabled(false);
		prevAction.setEnabled(false);		
		this.setTitle(xmlTitle+" ("+ list.size()+" occurrences of \""+ ts.getSearchString()+"\" )");
	}
	
	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public void addRow(Object row) {
		if (row!=null) 
			viewer.add(row);
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize it.
	 * 
	 */
	public void createPartControl(Composite parent) {
		int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | 
					SWT.FULL_SELECTION | SWT.HIDE_SELECTION;		
		viewer = new TableViewer(parent, style); //SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.HIDE_SELECTION);
		Table table=(Table)viewer.getControl();
		table.setLinesVisible (true);
		table.setHeaderVisible (true);
		String[] titles = {"!", "Name", "Occurrence", "Description" };
		
		TableColumn column = new TableColumn (table, SWT.CENTER,0);
		column.setText ("!");
		column.setWidth (20);
		column = new TableColumn (table, SWT.LEFT,1);
		column.setText ("Name");
		column.setWidth (250);
		column = new TableColumn (table, SWT.LEFT,2);
		column.setText ("Occurrence");
		column.setWidth (100);
		column = new TableColumn (table, SWT.CENTER,3);
		column.setText ( "Description");
		column.setWidth (300);

		viewer.setUseHashlookup(true);		
		viewer.setColumnProperties(titles);

		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.addSelectionChangedListener(viewSelectionChangedListener);
		viewer.setInput(list);
		
		makeActions();
		hookContextMenu();
		contributeToActionBars();
		hookDoubleClickAction();
		xmlTitle=this.getTitle();
	}


	// Creates actiosn defiend in the class
	protected void makeActions() {
		doubleClickAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection)selection).getFirstElement();
				IStructuredSelection jumpto=(IStructuredSelection)obj;
//				TraceViewer tv=(TraceViewer)search.getFocusedEditor().getActiveViewer();
				IEditorPart ed=SeatUtil.getActiveEditor();
				if (ed instanceof ITraceEditor) {
					search.setEditor((ITraceEditor)ed);
					TraceViewer tv=(TraceViewer)((ITraceEditor)ed).getActiveViewer();
					tv.setSelection(jumpto,true);
				
					// test if it's first
					IStructuredSelection first=(IStructuredSelection)viewer.getElementAt(0);
					if ((first!=null)&&(jumpto==first)) {
						firstAction.run();
//						lastAction.setEnabled(true);
//						nextAction.setEnabled(true);
//						prevAction.setEnabled(true);
					}
				}					
			}
		};			
				
		
		// TODO Auto-generated method stub
		firstAction = new Action() {
			public void run() {
				IEditorPart ed=SeatUtil.getActiveEditor();
				if (ed instanceof ITraceEditor) {
					search.setEditor((ITraceEditor)ed);
					search.setResponsive(true);
					search.searchFirst();
					lastAction.setEnabled(true);
					nextAction.setEnabled(true);
					prevAction.setEnabled(true);
				}
			}
		};
		firstAction.setText("Go to First Occurrence");
		firstAction.setToolTipText("Go to First Occurrence");
		firstAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/first.gif"));
		firstAction.setEnabled(false);
		
		lastAction = new Action() {
			public void run() {
				IEditorPart ed=SeatUtil.getActiveEditor();
				if (ed instanceof ITraceEditor) {
					search.setEditor((ITraceEditor)ed);
					search.setResponsive(true);
					search.searchLast(false);
				}
			}
		};
		lastAction.setText("Go to Last Occurrence");
		lastAction.setToolTipText("Go to Last Occurrence");
		lastAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/last.gif"));
		lastAction.setEnabled(false);

		nextAction = new Action() {
			public void run() {
				IEditorPart ed=SeatUtil.getActiveEditor();
				if (ed instanceof ITraceEditor) {
					search.setEditor((ITraceEditor)ed);
					search.setResponsive(true);
					search.searchNext();
				}
			}
		};
		nextAction.setText("Go to Next Occurrence");
		nextAction.setToolTipText("Go to Next Occurrence");
		nextAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/right.gif"));
		nextAction.setEnabled(false);
		
		prevAction = new Action() {
			public void run() {
				IEditorPart ed=SeatUtil.getActiveEditor();
				if (ed instanceof ITraceEditor) {
					search.setEditor((ITraceEditor)ed);
					search.setResponsive(true);
					search.searchPrevious();
				}
			}
		};
		prevAction.setText("Go to Previous Occurrence");
		prevAction.setToolTipText("Go to Previous Occurrence");
		prevAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/left.gif"));	
		prevAction.setEnabled(false);
	}

	// Adds context menu
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenuExpl");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				SearchView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	// Adds double click funtion to the viewer
	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (doubleClickAction!=null)
					doubleClickAction.run();
			}
		});
	}

	// Adds items to the pulldown menu and toolbar of this view
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	// Hooks to the pulldown, called by hookcontextMenu
	protected void fillContextMenu(IMenuManager manager) {
		manager.add(firstAction);
		manager.add(prevAction);
		manager.add(nextAction);
		manager.add(lastAction);
		manager.add(new Separator("Additions")); // a must for context menu
	}	
	// Adds actions to the local pulldown menu
	protected void fillLocalPullDown(IMenuManager manager) {}

	// Adds actions to local tool bar 
	protected void fillLocalToolBar(IToolBarManager manager) {
		manager.add(firstAction);
		manager.add(prevAction);
		manager.add(nextAction);
		manager.add(lastAction);		
	}

}
