/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package seat.ui.views;

import java.util.ArrayList;
import java.util.Arrays;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableTreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import seat.SeatUtil;
import seat.io.TraceState;
import seat.model.core.dynamic.Trace;
import seat.ui.tree.ITraceEditor;

/**
 * This sample class demonstrates how to plug-in a new
 * workbench view. The view shows data obtained from the
 * model. The sample creates a dummy model on the fly,
 * but a real implementation would connect to the model
 * available either in this or another plug-in (e.g. the workspace).
 * The view is connected to the model using a content provider.
 * <p>
 * The view uses a label provider to define how model
 * objects should be presented in the view. Each
 * view can present the same model objects using
 * different labels and icons, if needed. Alternatively,
 * a single label provider can be shared between views
 * in order to ensure that objects of the same type are
 * presented in the same way everywhere.
 * <p>
 */

public class StatisView extends ViewPart implements IPartListener{
	private TableTreeViewer viewer;
	private Trace trace;
	private Action action1;
	private Action action2;
	private Action doubleClickAction;

	private ITraceEditor previous=null;
	/*
	 * The content provider class is responsible for
	 * providing objects to the view. It can wrap
	 * existing objects in adapters or simply return
	 * objects as-is. These objects may be sensitive
	 * to the current input of the view, or ignore
	 * it and always show the same content 
	 * (like Task List, for example).
	 */
	class Statis {
		String name;
		String value;
		Statis parent;
		ArrayList children=new ArrayList(1); 
		Statis(String n,String v, Statis p) {
			name=n;value=v;parent=p;
			if (parent!=null) parent.children.add(this); // parent of inv-root is null 
		}
	}
	
	class ViewContentProvider implements ITreeContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		public Object[] getElements(Object parent) {
			return getChildren(parent);
		}
		public Object getParent(Object child) {
			return ((Statis) child).parent;
		}
		public Object[] getChildren(Object parent) {
			return ((Statis)parent).children.toArray(new Statis[((Statis)parent).children.size()]);
		}
		public boolean hasChildren(Object parent) {
			return ((Statis) parent).children.size()>0;
		}		
		
	}
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			if (obj instanceof Statis) {
				Statis s=(Statis)obj;
				switch (index) {
					case 0:
						return s.name;
					case 1:
						return s.value;
					default:
						return "";
					}
			} else
				return obj.toString();	
		}
		
		public Image getColumnImage(Object element, int columnIndex) {
			// TODO Auto-generated method stub
			return null;
		}

	}
	class NameSorter extends ViewerSorter {
	}

	/**
	 * The constructor.
	 */
	public StatisView() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
		super.dispose();
		this.getSite().getPage().removePartListener(this);
	}
	
	public void setTrace(Trace trace){
		this.trace=trace;
	}	
	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		viewer = new TableTreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		Table table=viewer.getTableTree().getTable();
		table.setLinesVisible (true);
		table.setHeaderVisible (true);
		String[] titles = {"Property", "Value"};
		
		TableColumn column = new TableColumn (table, SWT.CENTER,0);
		column.setText (titles[0]);
		column.setWidth (250);
		column = new TableColumn (table, SWT.LEFT,1);
		column.setText (titles[1]);
		column.setWidth (250);

		viewer.setUseHashlookup(true);		
		viewer.setColumnProperties(titles);

		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
//		viewer.setSorter(new NameSorter());
		viewer.setInput(getTree());
//		makeActions();
//		hookContextMenu();
//		hookDoubleClickAction();
//		contributeToActionBars();
		// add self as the listener to the editor 
		this.getSite().getPage().addPartListener(this);				
	}

//	private void hookContextMenu() {
//		MenuManager menuMgr = new MenuManager("#PopupMenu");
//		menuMgr.setRemoveAllWhenShown(true);
//		menuMgr.addMenuListener(new IMenuListener() {
//			public void menuAboutToShow(IMenuManager manager) {
//				StatisView.this.fillContextMenu(manager);
//			}
//		});
//		Menu menu = menuMgr.createContextMenu(viewer.getControl());
//		viewer.getControl().setMenu(menu);
//		getSite().registerContextMenu(menuMgr, viewer);
//	}
//
//	private void contributeToActionBars() {
//		IActionBars bars = getViewSite().getActionBars();
//		fillLocalPullDown(bars.getMenuManager());
//		fillLocalToolBar(bars.getToolBarManager());
//	}
//
//	private void fillLocalPullDown(IMenuManager manager) {
//		manager.add(action1);
//		manager.add(new Separator());
//		manager.add(action2);
//	}
//
//	private void fillContextMenu(IMenuManager manager) {
//		manager.add(action1);
//		manager.add(action2);
//		// Other plug-ins can contribute there actions here
//		manager.add(new Separator("Additions"));
//	}
//	
//	private void fillLocalToolBar(IToolBarManager manager) {
//		manager.add(action1);
//		manager.add(action2);
//	}
//
//	private void makeActions() {
//		action1 = new Action() {
//			public void run() {
//				showMessage("Action 1 executed");
//			}
//		};
//		action1.setText("Action 1");
//		action1.setToolTipText("Action 1 tooltip");
//		action1.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
//			getImageDescriptor(ISharedImages.IMG_OBJS_INFO_TSK));
//		
//		action2 = new Action() {
//			public void run() {
//				showMessage("Action 2 executed");
//			}
//		};
//		action2.setText("Action 2");
//		action2.setToolTipText("Action 2 tooltip");
//		action2.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
//			getImageDescriptor(ISharedImages.IMG_OBJS_TASK_TSK));
//		doubleClickAction = new Action() {
//			public void run() {
//				ISelection selection = viewer.getSelection();
//				Object obj = ((IStructuredSelection)selection).getFirstElement();
//				showMessage("Double-click detected on "+obj.toString());
//			}
//		};
//	}
//
//	private void hookDoubleClickAction() {
//		viewer.addDoubleClickListener(new IDoubleClickListener() {
//			public void doubleClick(DoubleClickEvent event) {
//				doubleClickAction.run();
//			}
//		});
//	}
//	private void showMessage(String message) {
//		MessageDialog.openInformation(
//			viewer.getControl().getShell(),
//			"Sample View",
//			message);
//	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		Control viewerControl=viewer.getControl();
		if (viewerControl!=null && !viewerControl.isDisposed()) 
			viewerControl.setFocus();
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partActivated(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partActivated(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		if ((part instanceof ITraceEditor)&& (part!=previous)) {
			previous=(ITraceEditor)part;
			setTrace(((ITraceEditor)part).getTrace());
			viewer.setInput(getTree());
			viewer.refresh();
		} 
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partBroughtToTop(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partBroughtToTop(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		partActivated(part);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partClosed(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partClosed(IWorkbenchPart part) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partDeactivated(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partDeactivated(IWorkbenchPart part) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partOpened(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partOpened(IWorkbenchPart part) {
		// TODO Auto-generated method stub

	}
	private Statis getTree() {
		Statis inv=new Statis("inv","inv",null);
		if (trace==null) return inv;
		TraceState state=trace.getState();
		Statis t10=new Statis("Total Nodes",state.getTreeNodes()+"",inv);
		Statis t11=new Statis("Nodes in Graph",state.getGraphSize()+"",inv);
		Statis t12=new Statis("Nodes Compressed",state.getCompressedNodes()+"",inv);
		Statis t13=new Statis("Loading Time",state.getLoadTime()+"sec",inv);
		Statis t14=new Statis("Pattern Detected",state.getDetectedPatterns()+"",inv);
		Statis t15=new Statis("Trace Source",state.getSource().getName(),inv);
		int type=state.getType();
		String st;
		switch (type) {
			case SeatUtil.TRACE_C:
				st="C";
			case SeatUtil.TRACE_JAVA:
				st="JAVA";
			case SeatUtil.TRACE_OO:
				st="Object Oriented";
			case SeatUtil.TRACE_ROUTINE:
				st="Routine";
			default:
				st="Undetermined";
		}		
		Statis t16=new Statis("Tace Type",st,inv);
		Statis t17=new Statis("Total Packages",state.getPackageList().size()+"",inv);
		Statis t18=new Statis("Total classes",state.getClassList().size()+"",inv);
			
		
		ArrayList plu=state.getPackageList();
		String[] plus=(String[])plu.toArray(new String[plu.size()]);
		Arrays.sort(plus);
		for (int i=0;i<plus.length;i++)
			new Statis(plus[i],"",t17);			
		ArrayList clu=state.getClassList();
		String[] clus=(String[])clu.toArray(new String[clu.size()]);
		Arrays.sort(clus);
		for (int i=0;i<clus.length;i++)
			new Statis(clus[i],"",t18);	
			
		return inv;				
	}
	
	public void refresh() {
		viewer.setInput(getTree());
		viewer.refresh();		
	}
}