/*
 * Created on Nov 26, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.ui.views;

import java.util.ArrayList;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;

import seat.ui.algorithm.NameParsing;
import seat.ui.tree.ITraceEditor;
import seat.ui.tree.ITreeNode;
import seat.ui.tree.TraceViewer;

/**
 * @author lfu
 * Searches a UI tree given in a tree editor. Two parameters can be set prior to the begin of a search
 * Exact: for exact match; Responsive for if the earch search action will be reflect in UI(set current selection).
 * 1. For gather all the search result, first needs to call setResponsive(false) so the UI do not active;
 *    Then gatherAllMatched() can be called to do the search; 
 *    And getSearch() to return the results in a list
 * 2. Search can also be done one by one in a tree, so there are four corresponding method:
 * 	  searchFirst, searchLast, searchNext, searchPrev(); before calling them, the search string must be
 *    set FIRST by calling searchFirst(String).  searchLast is also used by gathering results.
 *    Actual search is done in search() method which return one match per run. 
 */

// data is structuredselection;
public class TraceSearch {
	private ITraceEditor ed;
	private TraceViewer viewer;
	private IStructuredSelection currentSelection;
	private String searchString; 
	private boolean continueSearch=true;
	private ITreeNode result; 
	
	private boolean responsive=true; // if to set current selectino to new one
	private boolean exactly=false;  // exact match
	
	private ArrayList lastSearch=new ArrayList();
	/**
	 * @param parent
	 */
	public TraceSearch(ITraceEditor editor) {
		setEditor(editor);
		if (((IStructuredSelection)viewer.getSelection()).getFirstElement()==null) { // set sele to first node
			ITreeNode root=viewer.getTraceTree().getInvisibleRoot().getChildren()[0];
			jumpToSelection(new StructuredSelection(root));						
		}
	}
	
	public void setEditor(ITraceEditor editor) {
		this.ed=editor;
		this.viewer=(TraceViewer)editor.getActiveViewer();
	}
	// UI responsiveness
	public void setResponsive(boolean res) {
		responsive=res;
	}
	
	// Exact match or not
	public void setExact(boolean exact) {
		exactly=exact;
	}		

	// Gets all the searches and put them in the list
	public void gatherAllMatched(String name, boolean exact) {
		responsive=false;
		exactly=exact;
		searchFirst(name);
		searchLast(true);
	}
	// Gets all the searches and put them in the list
	// name can be method name, fully qualified name, depending on user input 
	public void gatherAllMatched(String name, boolean exact,IProgressMonitor monitor) {
		responsive=false;
		exactly=exact;
		searchFirst(name);
		monitor.worked(5);
		searchLast(true);
		monitor.worked(95);
	}
	
	// Returns the research result
	public ArrayList getSearch() {
		return lastSearch;
	}

	/**
	 * Also be first method to be called when starting a search, because the search string and
	 * status needs to be set by it. gatherAllMatched() method call this method automatically. 
	 * @param search-method nmae string, do not include package/class name
	 */
	public void searchFirst(String search) {
		lastSearch.clear();  //for temp. clear search remember list
		this.searchString=search;
		currentSelection=(IStructuredSelection)viewer.getSelection();
		ITreeNode root=viewer.getTraceTree().getInvisibleRoot().getChildren()[0];
		ITreeNode result=search(searchString,root,true);
		if (result!=null) {
			currentSelection=new StructuredSelection(result);
			if (responsive)
				jumpToSelection(currentSelection);
			continueSearch=true;
			addToSearchList(currentSelection);
		} else
			continueSearch=false;
	}

	// Searches first again after some time 
	public void searchFirst() {
		if (searchString==null)
			return;
		currentSelection=(IStructuredSelection)viewer.getSelection();
		ITreeNode root=viewer.getTraceTree().getInvisibleRoot().getChildren()[0];
		ITreeNode result=search(searchString,root,true);
		if (result!=null) {
			currentSelection=new StructuredSelection(result);
			if (responsive) 
				jumpToSelection(currentSelection);
			continueSearch=true;
			addToSearchList(currentSelection);
		} else
			continueSearch=false;
	}

	// search last match
	public void searchLast(boolean gathering) {
		//this.getControl().setRedraw(false);
		// first search may get result, so save it
		ITreeNode found=null;
		search4Matches(true);
		if (result!=null) { 		
			found=result;
			if (gathering)
				addToSearchList(new StructuredSelection(result));			
		}
		while (continueSearch) {
			search4Matches(false);
			if (result!=null) {  		
				found=result;
				if (gathering)
					addToSearchList(new StructuredSelection(result));			
			}
		}
		if (result!=null) {
			currentSelection=new StructuredSelection(result);
			if (responsive) 
				jumpToSelection(currentSelection);
			addToSearchList(currentSelection);
		} else 	if (found!=null) {
			currentSelection=new StructuredSelection(found);
			if (responsive) 
				jumpToSelection(currentSelection);
			addToSearchList(currentSelection);
		}
	}

	/** used by search last to skip one by one till last node that is finished tree 
	 * @param firstTime
	 */
	private void search4Matches(boolean firstTime) {
		if (!continueSearch)
			return;
		ITreeNode root=viewer.getTraceTree().getInvisibleRoot().getChildren()[0];
		ITreeNode current;
		if (firstTime) {
			currentSelection=(IStructuredSelection)viewer.getSelection();
			current=(ITreeNode)currentSelection.getFirstElement();
		} else {
			current=result;
		}
		if (current==null) 
			return;
		ITreeNode prev=current;
		do { 
			if (prev==current)
				result=search(searchString,current,false);  // in children
			else
				result=search(searchString,current,true);  // in curernt and children			 
			if (result!=null) {
				continueSearch=true;
				return;
			}
			ITreeNode sibling=current.getNextSibling();
			ITreeNode parent=current.getParent();
			if (sibling!=null)  // search sibling
				current=sibling;
			else if (sibling==null&&parent==root) { // test for end
				continueSearch=false;
				return;					
			} else { // now cond is: parent!=root
				current=parent.getNextSibling();
				while (current==null) {
					parent=parent.getParent();
					if (parent==null) {
						continueSearch=false;
						return;											
					}
					current=parent.getNextSibling();										
					if (current==parent&&parent==root) {
						continueSearch=false;
						return;											
					}
				}				
			}
		} while (true);
	}

	// search for next match
	public void searchNext() {
		if (!continueSearch)
			return;
		ITreeNode root=viewer.getTraceTree().getInvisibleRoot().getChildren()[0];
		currentSelection=(IStructuredSelection)viewer.getSelection();
		ITreeNode current=(ITreeNode)currentSelection.getFirstElement();
		if (current==null) 
			return;
		ITreeNode prev=current,result;
		do { 
			if (prev==current)
				result=search(searchString,current,false);  // in children
			else
				result=search(searchString,current,true);  // in curernt and children			 
			if (result!=null) {
				currentSelection=new StructuredSelection(result);
				if (responsive)
					jumpToSelection(currentSelection);
				continueSearch=true;
				addToSearchList(currentSelection);
				return;
			}
			ITreeNode sibling=current.getNextSibling();
			ITreeNode parent=current.getParent();
			if (sibling!=null)  // search sibling
				current=sibling;
			else if (sibling==null&&parent==root) { // test for end
				continueSearch=false;
				return;					
			} else { // now cond is: parent!=root
				if (parent==null)
					return;
				current=parent.getNextSibling();
				while (current==null) {
					parent=parent.getParent();
					if (parent==null) {
						continueSearch=false;
						return;											
					}
					current=parent.getNextSibling();										
					if (current==parent&&parent==root) {
						continueSearch=false;
						return;											
					}
				}				
			}
		} while (true);
	}

	// search for a previous match
	public void searchPrevious() {
		if (lastSearch.size()!=0) {
			currentSelection=(IStructuredSelection)viewer.getSelection();
			int cur=lastSearch.indexOf(currentSelection);
			int prev;
			if (cur!=-1) {  //found
				prev=(cur==0)?0:cur-1; 	
				currentSelection=(IStructuredSelection)lastSearch.get(prev);
				if (responsive)
					jumpToSelection(currentSelection);
				continueSearch=true;
			}
		}
	}
	
	// search from the current node for a match; if cannot find, return null
	// s will be copmared with name of from. Meaning of s and from should be taken care. 
	private ITreeNode search(String s,ITreeNode from,boolean includeCurrent) {
		if (from==null)
			return null;
		ITreeNode found;
		String methodName=from.getName();
		if (includeCurrent)
			if (exactly) {
				if (methodName.equals(s)) {
					return from;
				}
			} else {
				if (methodName.lastIndexOf(s)!=-1) {
					return from;
				}				
			}
		if (from==null)
			return null;
		ITreeNode[] children=from.getChildren();
		for (int i=0;i<children.length;i++) {
			methodName=children[i].getName();			
			if (exactly) {
				if (methodName.equals(s))
					return children[i];
				else {
					found=search(s,children[i],true);
					if (found!=null)
						return found;
				}				
			} else {
				if (methodName.lastIndexOf(s)!=-1)
					return children[i];
				else {
					found=search(s,children[i],true);
					if (found!=null)
						return found;
				}
			}
		}
		return null;
	}

	private void jumpToSelection(IStructuredSelection jump) {
		viewer.getControl().setRedraw(false);
		viewer.setSelection(jump,true);
		viewer.getControl().setRedraw(true);						
	}
	
	
	// temp add guard, will further test for search
	private void addToSearchList(IStructuredSelection sel) {
		if (!lastSearch.contains(sel)) {
//			ITreeNode current=(ITreeNode)currentSelection.getFirstElement();
//			String methodName=NameParsing.getShortMethodName(current.getElement());
//			if (methodName.indexOf(searchString)!=-1)
				lastSearch.add(sel);				
		}
	}
	
	// help method
	public ITraceEditor getFocusedEditor() {
		return ed;
	}
	
	// help method
	public String getSearchString() {
		return searchString;
	}
}

