/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package seat.ui.views;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.search.ui.IActionGroupFactory;
import org.eclipse.search.ui.IGroupByKeyComputer;
import org.eclipse.search.ui.ISearchResultView;
import org.eclipse.search.ui.ISearchResultViewEntry;
import org.eclipse.search.ui.SearchUI;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;
import seat.ui.tree.ITraceEditor;
import seat.ui.tree.TraceViewer;

/**
 * Help Search Operation.
 */
public class TraceSearchOperation extends WorkspaceModifyOperation {
	// Images
	private static final ImageDescriptor IMAGE_DSCR_SEARCH =
		SeatUtil.getImageDescriptor("icons/class_obj.gif");
//		ImageDescriptor.createFromURL(
//			WorkbenchResources.getImagePath(
//				IHelpUIConstants.IMAGE_FILE_SEARCH));
//	private static final ImageDescriptor IMAGE_DSCR_TOPIC =
//		ImageDescriptor.createFromURL(
//			WorkbenchResources.getImagePath(IHelpUIConstants.IMAGE_FILE_TOPIC));
//	// Resource we will be using
	private static final IResource resource =
		ResourcesPlugin.getWorkspace().getRoot();
	IProgressMonitor fMonitor;
	private static ImageRegistry imgRegistry = null;
	private String queryData = null;
	private ArrayList nodeResults=new ArrayList(40);
	private boolean exactMatch;
	
	private TraceSearch searchResult;
	/**
	 * TraceSearchOperation constructor.
	 * @param data SearchQueryData
	 */
	public TraceSearchOperation(String searchString,boolean exact, TraceSearch searchResult) {
//		if (imgRegistry == null) {
//			imgRegistry = WorkbenchHelpPlugin.getDefault().getImageRegistry();
//			imgRegistry.put(
//				IHelpUIConstants.IMAGE_KEY_SEARCH,
//				IMAGE_DSCR_SEARCH);
//			imgRegistry.put(IHelpUIConstants.IMAGE_KEY_TOPIC, IMAGE_DSCR_TOPIC);
//		}
		queryData = searchString;
		this.exactMatch=exact;
		this.searchResult=searchResult;
	}
	/**
	 * @see WorkspaceModifyOperation#execute(IProgressMonitor)
	 */
	protected void execute(IProgressMonitor monitor)
		throws CoreException, InvocationTargetException, InterruptedException {
		IProgressMonitor fMonitor=monitor;
		monitor.beginTask("Begin Searching",100);
		try {
			doSearch();
//			searchTrace();			
//			displayResults();
		} catch (OperationCanceledException oce) {
			// allowed, no logging
			monitor.done();
			throw oce;
		} catch (Exception e) {
			e.printStackTrace();
			monitor.done();
		}
		monitor.done();
	}
	
	private void doSearch() {
//		IEditorPart active=SeatUtil.getOpenedEditor();
//		if (active instanceof ITraceEditor) {
//			TraceViewer tv=(TraceViewer)((ITraceEditor)active).getViewer();
//			searchResult.gatherAllMatched(queryData,exactMatch);
//		}
		searchResult.gatherAllMatched(queryData,exactMatch);
	}
//	SearchView sv=(SearchView)SeatUtil.findOpenedView("seat.ui.views.SearchView");
//	sv.setSearch(ts);
	
	private void searchTrace() {
		IEditorPart active=SeatUtil.getOpenedEditor();
		if (active instanceof ITraceEditor) {
			Trace trace=((ITraceEditor)active).getTrace();
			search(((TreeTrace)trace).getRoot());
		}
			
	}
	
	private void search(Node node) {
	   for(int i = 0; i < node.getNumberOfChildren(); i++)
	   {
		  Node child = (Node)node.getChildAt(i);
		  if (exactMatch) {
			if (child.getBehaviouralElement().getName().equals(queryData))
				nodeResults.add(child);
		  } else {
			if (child.getBehaviouralElement().getName().indexOf(queryData)!=-1)
				nodeResults.add(child);		  	
		  }
		  search(child);
	   }
	}
    	
	
	private void displayResults() {
		ISearchResultView sView = SearchUI.getSearchResultView();
		if (sView != null)
			sView.searchStarted(
					(IActionGroupFactory) null,
					"found 1 match","found {0} matches",
					IMAGE_DSCR_SEARCH,
					"seat.ui.views.TraceSearchPage",
					new LabelProvider() {
			public String getText(Object element) {
				if (element instanceof ISearchResultViewEntry)
					try {
						ISearchResultViewEntry entry =
							(ISearchResultViewEntry) element;
					 
						return (String)entry.getSelectedMarker().getAttribute("seat.ui.tree.label");
					} catch (CoreException ce) {
					}
				return "2";
			}
			public Image getImage(Object element) {
				if (element instanceof ISearchResultViewEntry)
					return IMAGE_DSCR_SEARCH.createImage();
				return null;
			}
		}, new org.eclipse.jface.action.Action() {
			public void run() {
				ISearchResultView view = SearchUI.getSearchResultView();
				view.getSelection();
				ISelection selection = view.getSelection();
				Object element = null;
				if (selection instanceof IStructuredSelection)
					element =
						((IStructuredSelection) selection).getFirstElement();
				if (element instanceof ISearchResultViewEntry) {
					ISearchResultViewEntry entry =
						(ISearchResultViewEntry) element;
					try {
						gotoEditor(entry);
					} catch (Exception e) {
						System.out.println(e);
					}
				}
			}
		}, new IGroupByKeyComputer() {
			public Object computeGroupByKey(IMarker marker) {
				try {
					if (marker.getAttribute("seat.ui.tree.label")!=null)
						return marker.getAttribute("seat.ui.tree.label");
				} catch (CoreException ce) {
				}
				return "UNKNOWN";
			}
		}, this);
		// Delete all previous results
		try {
			resource.deleteMarkers(
				"org.eclipse.core.resources.bookmark",
				true,
				IResource.DEPTH_INFINITE);
		} catch (CoreException ex) {
		}
		createResultsMarkers(sView);
		sView.searchFinished();
	}
	private void createResultsMarkers(ISearchResultView sView) {
		Node[] searchHits=(Node[])nodeResults.toArray(new Node[nodeResults.size()]);
		for (int i = 0; i < searchHits.length; i++) {
			try {
				IMarker marker = null;
				marker = resource.createMarker("seat.marker");
				marker.setAttribute(
					"seat.ui.tree.label",
					searchHits[i].getLabel());

				sView.addMatch(
					searchHits[i].getLabel(),
					searchHits[i].getLabel(),
					resource,
					marker);
			} catch (CoreException ce) {
			}
		}
	}
	
	private void gotoEditor(ISearchResultViewEntry entry) {
		
	}
}
