package seat.ui.views;

import java.util.Vector;

import seat.model.core.dynamic.Node;

/**
 * A TreeObject is the basic visual node that exists in a treeviewer.
 * It wraps a model Node and query it to get nessary infomation for
 * display in the tree. 
 * <p>
 * A TreeObject has a parent and a direct children vector. It also
 * implements the IAdaptable interface for the sake of peoperty display.
 * <p>
 * @author lfu
 * First created on May 31, 2003
 */
public class TreeObject {
	Node modelNode;
	private TreeObject parent; 
	private Vector children;
	
	private boolean expanded;
	
	/**
	 * Constructor to create a new TreeObject
	 * @param modelNode - model node that thid TreeObject stands for
	 * @param parent - parent of current, can be null for root 
	 */
	public TreeObject(Node model,TreeObject parent) {
		modelNode=model; // may be MethIn or RC
		children=new Vector();
		this.parent=parent;
		if ((parent!=null)&&parent.children!=null) 
			parent.children.add(this);
		expanded=false;
	}
	/**
	 * Returns the parent
	 * @return
	 */
	public TreeObject getParent() {
		return parent;
	}
	
	public TreeObject[] getChildren() {
		return (TreeObject[])children.toArray(new TreeObject[children.size()]);
	}
	
	/**
	 * Check the flag to determine if current TreeObject has children.
	 * Its does not really check children.size(), because the children
	 * is retreived dynamically. A TreeObject may has children but not
	 * loaded yet.
	 */ 
	public boolean hasChildren() {
		return modelNode.getNumberOfChildren()>0; // !!!!!! test for model, current maybe empty
	}

	
	public Node getModel() {
		return modelNode;
	}
	
	public String getName() {
		return modelNode.getLabel();
	}
	
}
